# -*- coding: utf-8 -*-
# @Author  : LG

from PyQt5 import QtWidgets, QtCore
from PyQt5.QtCore import QThread, pyqtSignal
from ISAT.ui.Converter_dialog import Ui_Dialog
from ISAT.scripts.isat import ISAT
from ISAT.scripts.coco import COCO
from ISAT.scripts.yolo import YOLO
from ISAT.scripts.labelme import LABELME
from ISAT.scripts.voc import VOC
from ISAT.scripts.voc_detection import VOCDetect
import os
import yaml
import imgviz


class Converter(QThread, ISAT):
    message = pyqtSignal(int, int, str)
    def __init__(self):
        super(Converter, self).__init__()
        self.cancel = False
        self.isat_json_root = None

    def run(self):
        raise NotImplementedError

    def load_from_isat(self):
        self.annos.clear()
        self.cates = ()
        json_files = [file for file in os.listdir(self.isat_json_root) if file.endswith('.json')]
        num_json_files = len(json_files)
        for index, file in enumerate(json_files):
            if self.cancel:
                self.message.emit(-1, -1, ' ' * 18 + '| -- Cancel --')
                return

            self.message.emit(index+1, num_json_files,
                              '{:>8d}/{:<8d} | Loading ISAT json {}'.format(index + 1, num_json_files, file))
            try:
                anno = self._load_one_isat_json(os.path.join(self.isat_json_root, file))
                self.annos[self.remove_file_suffix(file)] = anno
            except Exception as e:
                self.message.emit(-1, -1, ' ' * 18 + '| Error: {}.'.format(e))

        # load cates
        self.message.emit(-1, -1, ' ' * 18 + '| Loading cats.')
        if os.path.exists(os.path.join(self.isat_json_root, 'isat.yaml')):
            cates = []
            with open(os.path.join(self.isat_json_root, 'isat.yaml'), 'rb')as f:
                cfg = yaml.load(f.read(), Loader=yaml.FullLoader)
            for label in cfg.get('label', []):
                cates.append(label.get('name'))
            self.cates = tuple(cates)
        else:
            cates = set()
            for _, anno in self.annos.items():
                for obj in anno.objs:
                    cates.add(obj.category)
            cates = list(cates)
            cates.sort()
            self.cates = tuple(cates)
        self.message.emit(-1, -1, ' ' * 18 + '| Loaded.')
        return True

    def convert_to_isat(self):
        self.message.emit(-1, -1, ' ' * 18 + '| Start convert to ISAT.')
        num_annos = len(self.annos)

        for index, (name_without_suffix, anno) in enumerate(self.annos.items()):
            if self.cancel:
                self.message.emit(-1, -1, ' ' * 18 + '| -- Cancel --')
                return

            self.message.emit(index + 1, num_annos,
                              '{:>8d}/{:<8d} | Converting to {}'.format(index + 1, num_annos,
                                                                        name_without_suffix + 'json'))
            try:
                self._save_one_isat_json(anno,
                                         os.path.join(self.isat_json_root, '{}.json'.format(name_without_suffix)))

            except Exception as e:
                self.message.emit(-1, -1, ' ' * 18 + '| Error: {}.'.format(e))

        # 类别文件
        cmap = imgviz.label_colormap()
        cates = sorted(self.cates)
        categories = []
        for index, cat in enumerate(cates):
            r, g, b = cmap[index + 1]
            categories.append({
                'name': cat if isinstance(cat, str) else str(cat),
                'color': "#{:02x}{:02x}{:02x}".format(r, g, b)
            })
        s = yaml.dump({'label': categories})
        with open(os.path.join(self.isat_json_root, 'isat.yaml'), 'w') as f:
            f.write(s)

        return True


class COCOConverter(Converter, COCO):
    def __init__(self):
        super(COCOConverter, self).__init__()
        self.coco_json_path = None
        self.isat_json_root = None
        self.keep_crowd = False
        self.coco2isat = True

    def run(self):
        if self.coco_json_path is None or self.isat_json_root is None:
            return

        if self.coco2isat:
            self.coco_to_isat()
        else:
            self.isat_to_coco()

    def coco_to_isat(self):
        self.message.emit(-1, -1, ' ' * 18 + '| -- COCO to ISAT --')

        # load from coco
        self.message.emit(-1, -1, ' ' * 18 + '| Loading from COCO.')
        try:
            self.read_from_coco(self.coco_json_path)
            self.message.emit(-1, -1, ' ' * 18 + '| Loaded.')

        except Exception as e:
            self.message.emit(-1, -1, ' ' * 18 + '| Error: {}.'.format(e))
            return

        # convert to isat
        self.convert_to_isat()
        self.message.emit(-1, -1, ' ' * 18 + '| Finished.')
        return True

    def isat_to_coco(self):
        self.message.emit(-1, -1, ' ' * 18 + '| -- ISAT to COCO --')

        # load from isat
        self.load_from_isat()

        # convert to coco
        self.message.emit(-1, -1, ' ' * 18 + '| Saving COCO json to {}.'.format(self.coco_json_path))
        try:
            self.save_to_coco(self.coco_json_path)
        except Exception as e:
            self.message.emit(-1, -1, ' ' * 18 + '| Error: {}.'.format(e))
        self.message.emit(-1, -1, ' ' * 18 + '| Finished.')
        return True


class YOLOConverter(Converter, YOLO):
    def __init__(self):
        super(YOLOConverter, self).__init__()
        self.yolo_images_root = None
        self.yolo_txt_root = None
        self.yolo_category_file = None
        self.isat_json_root = None

        self.yolo2isat = True

    def run(self):
        if self.yolo2isat:
            if self.yolo_images_root is None or self.yolo_txt_root is None or self.isat_json_root is None:
                return
            self.yolo_to_isat()
        else:
            if self.yolo_txt_root is None or self.isat_json_root is None:
                return
            self.isat_to_yolo()

    def yolo_to_isat(self):
        self.message.emit(-1, -1, ' ' * 18 + '| -- YOLO to ISAT --')

        # load category file
        if self.yolo_category_file is not None:
            self.message.emit(-1, -1, ' ' * 18 + '| Loading YOLO categorys.')
            class_dict = {}
            with open(self.yolo_category_file, 'r') as f:
                lines = f.readlines()
                for index, line in enumerate(lines):
                    class_dict[index] = line.rstrip('\n')
                    self.message.emit(-1, -1, ' ' * 18 + '| {} : {}'.format(index, line.rstrip('\n')))
            self.message.emit(-1, -1, ' ' * 18 + '| Loaded.')
        else:
            class_dict = None

        # load from yolo
        self.message.emit(-1, -1, ' ' * 18 + '| Loading YOLO txts.')
        img_files = os.listdir(self.yolo_images_root)
        num_img_files = len(img_files)

        for index, img_name in enumerate(img_files):
            if self.cancel:
                self.message.emit(-1, -1, ' ' * 18 + '| -- Cancel --')
                return
            name_without_suffix = self.remove_file_suffix(img_name)
            txt_path = os.path.join(self.yolo_txt_root, name_without_suffix + '.txt')
            image_path = os.path.join(self.yolo_images_root, img_name)
            if not os.path.exists(txt_path):
                continue

            self.message.emit(index + 1, num_img_files,
                              '{:>8d}/{:<8d} | Loading from {}'.format(index + 1, num_img_files,
                                                                       name_without_suffix + '.txt'))
            try:
                anno = self._load_one_yolo_txt(image_path, txt_path, class_dict)
                self.annos[name_without_suffix] = anno
            except Exception as e:
                self.message.emit(-1, -1, ' ' * 18 + '| Error: {}.'.format(e))

        # cates
        if class_dict is not None:
            self.cates = tuple(class_dict.values())
        else:
            class_set = set()
            for _, anno in self.annos.items():
                for obj in anno.objs:
                    class_set.add(obj.category)
            class_set = list(class_set)
            class_set.sort()
            self.cates = tuple(class_set)

        # convert to isat
        self.convert_to_isat()
        self.message.emit(-1, -1, ' ' * 18 + '| Finished.')

        return True

    def isat_to_yolo(self):
        self.message.emit(-1, -1, ' ' * 18 + '| -- ISAT to YOLO --')

        # load from isat
        self.load_from_isat()

        # save to yolo
        cates_index_dict = {cat: index for index, cat in enumerate(self.cates)}
        with open(os.path.join(self.yolo_txt_root, 'classification.txt'), 'w') as f:
            for cat in self.cates:
                f.write('{}\n'.format(cat))
        num_annos = len(self.annos)
        for index, (name_without_suffix, anno) in enumerate(self.annos.items()):
            if self.cancel:
                self.message.emit(-1, -1, ' ' * 18 + '| -- Cancel --')
                return

            txt_path = os.path.join(self.yolo_txt_root, name_without_suffix + '.txt')
            try:
                self.message.emit(index + 1, num_annos,
                                  '{:>8d}/{:<8d} | Save yolo txt to {}'.format(index + 1, num_annos,
                                                                           name_without_suffix + '.txt'))
                self._save_one_yolo_txt(anno, txt_path, cates_index_dict)
            except Exception as e:
                self.message.emit(-1, -1, ' ' * 18 + '| Error: {}.'.format(e))
        self.message.emit(-1, -1, ' ' * 18 + '| Finished.')
        return True


class LABELMEConverter(Converter, LABELME):
    def __init__(self):
        super(LABELMEConverter, self).__init__()
        self.isat_json_root = None
        self.labelme_json_root = None
        self.labelme2isat = True

    def run(self):
        if self.labelme_json_root is None or self.isat_json_root is None:
            return
        if self.labelme2isat:
            self.labelme_to_isat()
        else:
            self.isat_to_labelme()


    def labelme_to_isat(self):
        self.message.emit(-1, -1, ' ' * 18 + '| -- ISAT to ISAT --')

        # load from labelme
        self.message.emit(-1, -1, ' ' * 18 + '| Loading from LABELME.')
        json_files = [file for file in os.listdir(self.labelme_json_root) if file.endswith('.json')]
        num_json_files = len(json_files)
        for index, file in enumerate(json_files):
            if self.cancel:
                self.message.emit(-1, -1, ' ' * 18 + '| -- Cancel --')
                return

            name_without_suffix = self.remove_file_suffix(file)

            self.message.emit(index + 1, num_json_files,
                              '{:>8d}/{:<8d} | Loading from {}'.format(index + 1, num_json_files,
                                                                       name_without_suffix + '.json'))
            try:
                anno = self._load_one_labelme_json(os.path.join(self.labelme_json_root, file))
                self.annos[name_without_suffix] = anno
            except Exception as e:
                self.message.emit(-1, -1, ' ' * 18 + '| Error: {}.'.format(e))

        # cats
        class_set = set()
        for _, anno in self.annos.items():
            for obj in anno.objs:
                class_set.add(obj.category)

        class_set = list(class_set)
        class_set.sort()
        self.cates = tuple(class_set)

        # save to isat
        self.convert_to_isat()
        self.message.emit(-1, -1, ' ' * 18 + '| Finished.')

        return True

    def isat_to_labelme(self):

        # load from isat
        self.load_from_isat()

        # save to labelme
        num_annos = len(self.annos.items())
        for index, (name_without_suffix, anno) in enumerate(self.annos.items()):
            if self.cancel:
                self.message.emit(-1, -1, ' ' * 18 + '| -- Cancel --')
                return

            json_path = os.path.join(self.labelme_json_root, name_without_suffix + '.json')
            self.message.emit(index + 1, num_annos,
                              '{:>8d}/{:<8d} | Save LABELME json to {}'.format(index + 1, num_annos,
                                                                       name_without_suffix + '.json'))
            try:
                self._save_one_labelme_json(anno, json_path)
            except Exception as e:
                self.message.emit(-1, -1, ' ' * 18 + '| Error: {}.'.format(e))

        self.message.emit(-1, -1, ' ' * 18 + '| Finished.')
        return True


class VOCConverter(Converter, VOC):
    def __init__(self):
        super(VOCConverter, self).__init__()
        self.isat_json_root = None
        self.voc_png_root = None

    def run(self):
        if self.isat_json_root is not None and self.voc_png_root is not None:
            self.isat_to_voc()

    def isat_to_voc(self):
        self.message.emit(-1, -1, ' ' * 18 + '| -- ISAT to VOC --')

        # load from isat
        self.load_from_isat()

        # cmap
        cmap = imgviz.label_colormap()

        if not self.is_instance:
            category_index_dict = {}
            with open(os.path.join(self.voc_png_root, 'classification.txt'), 'w') as f:
                for index, cate in enumerate(self.cates):
                    category_index_dict[cate] = index
                    f.write('{}\n'.format(cate))
        else:
            category_index_dict = None
        # save to voc
        num_annos = len(self.annos)
        for index, (name_without_suffix, anno) in enumerate(self.annos.items()):
            if self.cancel:
                self.message.emit(-1, -1, ' ' * 18 + '| -- Cancel --')
                return

            png_path = os.path.join(self.voc_png_root, name_without_suffix + '.png')
            try:
                self.message.emit(index + 1, num_annos,
                                  '{:>8d}/{:<8d} | Save voc png to {}'.format(index + 1, num_annos,
                                                                           name_without_suffix + '.png'))
                self._save_one_voc_png(anno, png_path, cmap, category_index_dict)
            except Exception as e:
                self.message.emit(-1, -1, ' ' * 18 + '| Error: {}.'.format(e))

        self.message.emit(-1, -1, ' ' * 18 + '| Finished.')
        return True


class VOCConverterForDetection(Converter, VOCDetect):
    def __init__(self):
        super(VOCConverterForDetection, self).__init__()
        self.isat_json_root = None
        self.voc_xml_root = None

    def run(self):
        if self.isat_json_root is not None and self.voc_xml_root is not None:
            self.isat_to_voc_for_detection()

    def isat_to_voc_for_detection(self):
        self.message.emit(-1, -1, ' ' * 18 + '| -- ISAT to VOC for Detection --')

        # load from isat
        self.load_from_isat()

        # save to xml
        num_annos = len(self.annos)
        for index, (name_without_suffix, anno) in enumerate(self.annos.items()):
            if self.cancel:
                self.message.emit(-1, -1, ' ' * 18 + '| -- Cancel --')
                return

            xml_path = os.path.join(self.voc_xml_root, name_without_suffix + '.xml')
            try:
                self.message.emit(index + 1, num_annos,
                                  '{:>8d}/{:<8d} | Save voc png to {}'.format(index + 1, num_annos,
                                                                           name_without_suffix + '.png'))
                self._save_one_voc_xml(anno, xml_path)

            except Exception as e:
                self.message.emit(-1, -1, ' ' * 18 + '| Error: {}.'.format(e))
        self.message.emit(-1, -1, ' ' * 18 + '| Finished.')
        return True


class ConverterDialog(QtWidgets.QDialog, Ui_Dialog):
    def __init__(self, parent, mainwindow):
        super(ConverterDialog, self).__init__(parent=parent)
        self.setupUi(self)
        self.mainwindow = mainwindow
        self.converter = None
        self.setWindowModality(QtCore.Qt.WindowModality.WindowModal)
        self.tabWidget.currentChanged.connect(self.tab_widget_current_changed)

        self.init_connect()

    def tab_widget_current_changed(self):
        if self.tabWidget.currentWidget() == self.tab_COCO:
            self.mainwindow.cfg['software']['current_converter'] = 'coco'
        elif self.tabWidget.currentWidget() == self.tab_YOLO:
            self.mainwindow.cfg['software']['current_converter'] = 'yolo'
        elif self.tabWidget.currentWidget() == self.tab_LABELME:
            self.mainwindow.cfg['software']['current_converter'] = 'labelme'
        elif self.tabWidget.currentWidget() == self.tab_VOC:
            self.mainwindow.cfg['software']['current_converter'] = 'voc'
        elif self.tabWidget.currentWidget() == self.tab_VOC_DETECTION:
            self.mainwindow.cfg['software']['current_converter'] = 'voc for detection'
        else:
            self.mainwindow.cfg['software']['current_converter'] = 'coco'

    def apply(self):
        self.tabWidget.setEnabled(False)
        self.pushButton_convert.setEnabled(False)
        self.progressBar.reset()
        self.textBrowser.clear()

        if self.tabWidget.currentWidget() == self.tab_COCO:
            # COCO
            if self.toolBox_coco.currentWidget() == self.toolbox_item_coco2isat:
                # coco2isat
                if self.lineEdit_coco2isat_coco_json_path.text() and self.lineEdit_coco2isat_isat_json_root.text():
                    self.converter = COCOConverter()
                    self.converter.message.connect(self.print_message)
                    self.converter.coco2isat = True
                    self.converter.coco_json_path = self.lineEdit_coco2isat_coco_json_path.text()
                    self.converter.isat_json_root = self.lineEdit_coco2isat_isat_json_root.text()
                    self.converter.keep_crowd = self.checkBox_coco2isat_keep_crowd.isChecked()

                    self.converter.run()
                else:
                    QtWidgets.QMessageBox.warning(self, '', '')

            elif self.toolBox_coco.currentWidget() == self.toolbox_item_isat2coco:
                # isat2coco
                if self.lineEdit_isat2coco_isat_json_root.text() and self.lineEdit_isat2coco_coco_json_path.text():
                    self.converter = COCOConverter()
                    self.converter.message.connect(self.print_message)
                    self.converter.coco2isat = False
                    self.converter.isat_json_root = self.lineEdit_isat2coco_isat_json_root.text()
                    self.converter.coco_json_path = self.lineEdit_isat2coco_coco_json_path.text()
                    self.converter.keep_crowd = self.checkBox_isat2coco_keep_crowd.isChecked()

                    self.converter.run()
                else:
                    QtWidgets.QMessageBox.warning(self, '', '')

        elif self.tabWidget.currentWidget() == self.tab_YOLO:
            # YOLO
            if self.toolBox_yolo.currentWidget() == self.toolbox_item_yolo2isat:
                # yolo2isat
                if self.lineEdit_yolo2isat_yolo_image_root.text() and self.lineEdit_yolo2isat_yolo_txt_root.text() and self.lineEdit_yolo2isat_isat_json_root.text():
                    self.converter = YOLOConverter()
                    self.converter.message.connect(self.print_message)
                    self.converter.yolo2isat = True
                    self.converter.yolo_images_root = self.lineEdit_yolo2isat_yolo_image_root.text()
                    self.converter.yolo_txt_root = self.lineEdit_yolo2isat_yolo_txt_root.text()
                    self.converter.isat_json_root = self.lineEdit_yolo2isat_isat_json_root.text()
                    if self.lineEdit_yolo2isat_yolo_cate_path.text():
                        self.converter.yolo_category_file = self.lineEdit_yolo2isat_yolo_cate_path.text()

                    self.converter.run()
                else:
                    QtWidgets.QMessageBox.warning(self, '', '')

            elif self.toolBox_yolo.currentWidget() == self.toolbox_item_isat2yolo:
                # isat2yolo
                if self.lineEdit_isat2yolo_isat_json_root.text() and self.lineEdit_isat2yolo_yolo_txt_root.text():
                    self.converter = YOLOConverter()
                    self.converter.message.connect(self.print_message)
                    self.converter.yolo2isat = False
                    self.converter.isat_json_root = self.lineEdit_isat2yolo_isat_json_root.text()
                    self.converter.yolo_txt_root = self.lineEdit_isat2yolo_yolo_txt_root.text()

                    self.converter.run()
                else:
                    QtWidgets.QMessageBox.warning(self, '', '')

        elif self.tabWidget.currentWidget() == self.tab_LABELME:
            # LABELME
            if self.toolBox_labelme.currentWidget() == self.toolbox_item_labelme2isat:
                # labelme2isat
                if self.lineEdit_labelme2isat_labelme_json_root.text() and self.lineEdit_labelme2isat_isat_json_root.text():
                    self.converter = LABELMEConverter()
                    self.converter.message.connect(self.print_message)
                    self.converter.labelme2isat = True
                    self.converter.labelme_json_root = self.lineEdit_labelme2isat_labelme_json_root.text()
                    self.converter.isat_json_root = self.lineEdit_labelme2isat_isat_json_root.text()

                    self.converter.run()
                else:
                    QtWidgets.QMessageBox.warning(self, '', '')

            elif self.toolBox_labelme.currentWidget() == self.toolbox_item_isat2labelme:
                # isat2labelme
                if self.lineEdit_isat2labelme_isat_json_root.text() and self.lineEdit_isat2labelme_labelme_json_root.text():
                    self.converter = LABELMEConverter()
                    self.converter.message.connect(self.print_message)
                    self.converter.labelme2isat = False
                    self.converter.isat_json_root = self.lineEdit_isat2labelme_isat_json_root.text()
                    self.converter.labelme_json_root = self.lineEdit_isat2labelme_labelme_json_root.text()

                    self.converter.run()
                else:
                    QtWidgets.QMessageBox.warning(self, '', '')

        elif self.tabWidget.currentWidget() == self.tab_VOC:
            # VOC
            if self.lineEdit_isat2voc_isat_json_root.text() and self.lineEdit_isat2voc_voc_png_root.text():
                self.converter = VOCConverter()
                self.converter.message.connect(self.print_message)
                self.converter.isat_json_root = self.lineEdit_isat2voc_isat_json_root.text()
                self.converter.voc_png_root = self.lineEdit_isat2voc_voc_png_root.text()
                self.converter.is_instance = self.checkBox_is_instance.isChecked()
                self.converter.run()
            else:
                QtWidgets.QMessageBox.warning(self, '', '')

        elif self.tabWidget.currentWidget() == self.tab_VOC_DETECTION:
            # VOC_DETECTION
            if self.lineEdit_isat2vocod_isat_json_root.text() and self.lineEdit_isat2vocod_voc_xml_root.text():
                self.converter = VOCConverterForDetection()
                self.converter.message.connect(self.print_message)
                self.converter.isat_json_root = self.lineEdit_isat2vocod_isat_json_root.text()
                self.converter.voc_xml_root = self.lineEdit_isat2vocod_voc_xml_root.text()
                self.converter.run()
            else:
                QtWidgets.QMessageBox.warning(self, '', '')

        else:
            pass
        self.tabWidget.setEnabled(True)
        self.pushButton_convert.setEnabled(True)

    def cancel(self):
        try:
            self.converter.cancel = True
        except: pass

    def open_file(self):

        # coco2isat
        if self.sender() == self.pushButton_coco2isat_coco_json_path:
            filter="json (*.json)"
            lineEdit = self.lineEdit_coco2isat_coco_json_path
        # isat2coco
        elif self.sender() == self.pushButton_isat2coco_coco_json_path:
            filter = "json (*.json)"
            lineEdit = self.lineEdit_isat2coco_coco_json_path
        elif self.sender() == self.pushButton_yolo2isat_yolo_cate_path:
            filter = "txt (*.txt)"
            lineEdit = self.lineEdit_yolo2isat_yolo_cate_path
        else:
            filter = ''
            lineEdit = None

        path, suffix = QtWidgets.QFileDialog.getOpenFileName(self, caption='Open file', filter=filter)

        if path:
            if lineEdit is not None:
                lineEdit.setText(path)
        else:
            if lineEdit is not None:
                lineEdit.setText(path)

    def save_file(self):
        path, suffix = QtWidgets.QFileDialog.getSaveFileName(self, caption='Save file', filter="json (*.json)")
        # coco2isat
        if self.sender() == self.pushButton_coco2isat_coco_json_path:
            lineEdit = self.lineEdit_coco2isat_coco_json_path
        # isat2coco
        elif self.sender() == self.pushButton_isat2coco_coco_json_path:
            lineEdit = self.lineEdit_isat2coco_coco_json_path
        else:
            lineEdit = None

        if path:
            if not path.endswith('.json'):
                path += '.json'
            if lineEdit is not None:
                lineEdit.setText(path)
        else:
            if lineEdit is not None:
                lineEdit.clear()

    def open_dir(self):
        dir = QtWidgets.QFileDialog.getExistingDirectory(self, caption='Open dir')
        # coco2isat
        if self.sender() == self.pushButton_coco2isat_isat_json_root:
            lineEdit = self.lineEdit_coco2isat_isat_json_root
        # isat2coco
        elif self.sender() == self.pushButton_isat2coco_isat_json_root:
            lineEdit = self.lineEdit_isat2coco_isat_json_root
        # yolo2isat
        elif self.sender() == self.pushButton_yolo2isat_yolo_image_root:
            lineEdit = self.lineEdit_yolo2isat_yolo_image_root
        elif self.sender() == self.pushButton_yolo2isat_yolo_txt_root:
            lineEdit = self.lineEdit_yolo2isat_yolo_txt_root
        elif self.sender() == self.pushButton_yolo2isat_isat_json_root:
            lineEdit = self.lineEdit_yolo2isat_isat_json_root
        # isat2yolo
        elif self.sender() == self.pushButton_isat2yolo_isat_json_root:
            lineEdit = self.lineEdit_isat2yolo_isat_json_root
        elif self.sender() == self.pushButton_isat2yolo_yolo_txt_root:
            lineEdit = self.lineEdit_isat2yolo_yolo_txt_root
        # labelme2isat
        elif self.sender() == self.pushButton_labelme2isat_labelme_json_root:
            lineEdit = self.lineEdit_labelme2isat_labelme_json_root
        elif self.sender() == self.pushButton_labelme2isat_isat_json_root:
            lineEdit = self.lineEdit_labelme2isat_isat_json_root
        # isat2labelme
        elif self.sender() == self.pushButton_isat2labelme_isat_json_root:
            lineEdit = self.lineEdit_isat2labelme_isat_json_root
        elif self.sender() == self.pushButton_isat2labelme_labelme_json_root:
            lineEdit = self.lineEdit_isat2labelme_labelme_json_root
        # isat2voc
        elif self.sender() == self.pushButton_isat2voc_isat_json_root:
            lineEdit = self.lineEdit_isat2voc_isat_json_root
        elif self.sender() == self.pushButton_isat2voc_voc_png_root:
            lineEdit = self.lineEdit_isat2voc_voc_png_root
        # isat2vocod
        elif self.sender() == self.pushButton_isat2vocod_isat_json_root:
            lineEdit = self.lineEdit_isat2vocod_isat_json_root
        elif self.sender() == self.pushButton_isat2vocod_voc_xml_root:
            lineEdit = self.lineEdit_isat2vocod_voc_xml_root
        else:
            lineEdit = None

        if dir:
            if lineEdit is not None:
                lineEdit.setText(dir)
        else:
            if lineEdit is not None:
                lineEdit.clear()

    def save_dir(self):
        dir = QtWidgets.QFileDialog.getExistingDirectory(self, caption='Save dir')
        # coco2isat
        if self.sender() == self.pushButton_coco2isat_isat_json_root:
            lineEdit = self.lineEdit_coco2isat_isat_json_root
        # isat2coco
        elif self.sender() == self.pushButton_isat2coco_isat_json_root:
            lineEdit = self.lineEdit_isat2coco_isat_json_root
        # yolo2isat
        elif self.sender() == self.pushButton_yolo2isat_yolo_image_root:
            lineEdit = self.lineEdit_yolo2isat_yolo_image_root
        elif self.sender() == self.pushButton_yolo2isat_yolo_txt_root:
            lineEdit = self.lineEdit_yolo2isat_yolo_txt_root
        elif self.sender() == self.pushButton_yolo2isat_isat_json_root:
            lineEdit = self.lineEdit_yolo2isat_isat_json_root
        # isat2yolo
        elif self.sender() == self.pushButton_isat2yolo_isat_json_root:
            lineEdit = self.lineEdit_isat2yolo_isat_json_root
        elif self.sender() == self.pushButton_isat2yolo_yolo_txt_root:
            lineEdit = self.lineEdit_isat2yolo_yolo_txt_root
        # labelme2isat
        elif self.sender() == self.pushButton_labelme2isat_labelme_json_root:
            lineEdit = self.lineEdit_labelme2isat_labelme_json_root
        elif self.sender() == self.pushButton_labelme2isat_isat_json_root:
            lineEdit = self.lineEdit_labelme2isat_isat_json_root
        # isat2labelme
        elif self.sender() == self.pushButton_isat2labelme_isat_json_root:
            lineEdit = self.lineEdit_isat2labelme_isat_json_root
        elif self.sender() == self.pushButton_isat2labelme_labelme_json_root:
            lineEdit = self.lineEdit_isat2labelme_labelme_json_root
        # isat2voc
        elif self.sender() == self.pushButton_isat2voc_isat_json_root:
            lineEdit = self.lineEdit_isat2voc_isat_json_root
        elif self.sender() == self.pushButton_isat2voc_voc_png_root:
            lineEdit = self.lineEdit_isat2voc_voc_png_root
        # isat2vocod
        elif self.sender() == self.pushButton_isat2vocod_isat_json_root:
            lineEdit = self.lineEdit_isat2vocod_isat_json_root
        elif self.sender() == self.pushButton_isat2vocod_voc_xml_root:
            lineEdit = self.lineEdit_isat2vocod_voc_xml_root
        else:
            lineEdit = None

        if dir:
            self.save_root = dir
            if lineEdit is not None:
                lineEdit.setText(dir)
        else:
            if lineEdit is not None:
                lineEdit.clear()

    def print_message(self, index, all, message):
        if all > 0:
            self.progressBar.setMaximum(all)
        if index > 0:
            self.progressBar.setValue(index)
        if message:
            self.textBrowser.append(message)

    def init_connect(self):
        self.pushButton_convert.clicked.connect(self.apply)
        self.pushButton_cancel.clicked.connect(self.cancel)
        # coco2isat
        self.pushButton_coco2isat_coco_json_path.clicked.connect(self.open_file)
        self.pushButton_coco2isat_isat_json_root.clicked.connect(self.save_dir)
        # isat2coco
        self.pushButton_isat2coco_isat_json_root.clicked.connect(self.open_dir)
        self.pushButton_isat2coco_coco_json_path.clicked.connect(self.save_file)
        # yolo2isat
        self.pushButton_yolo2isat_yolo_image_root.clicked.connect(self.open_dir)
        self.pushButton_yolo2isat_yolo_txt_root.clicked.connect(self.open_dir)
        self.pushButton_yolo2isat_yolo_cate_path.clicked.connect(self.open_file)
        self.pushButton_yolo2isat_isat_json_root.clicked.connect(self.save_dir)
        # isat2yolo
        self.pushButton_isat2yolo_isat_json_root.clicked.connect(self.open_dir)
        self.pushButton_isat2yolo_yolo_txt_root.clicked.connect(self.save_dir)
        # labelme2isat
        self.pushButton_labelme2isat_labelme_json_root.clicked.connect(self.open_dir)
        self.pushButton_labelme2isat_isat_json_root.clicked.connect(self.save_dir)
        # isat2labelme
        self.pushButton_isat2labelme_isat_json_root.clicked.connect(self.open_dir)
        self.pushButton_isat2labelme_labelme_json_root.clicked.connect(self.save_dir)
        # voc
        self.pushButton_isat2voc_isat_json_root.clicked.connect(self.open_dir)
        self.pushButton_isat2voc_voc_png_root.clicked.connect(self.save_dir)
        # voc-object-detection
        self.pushButton_isat2vocod_isat_json_root.clicked.connect(self.open_dir)
        self.pushButton_isat2vocod_voc_xml_root.clicked.connect(self.save_dir)




# Copyright (c) Meta Platforms, Inc. and affiliates.
# All rights reserved.

# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

import torch

from functools import partial

from .modeling import ImageEncoderViT, MaskDecoder, PromptEncoder, Sam, TwoWayTransformer, TinyViT


def build_sam_vit_h(checkpoint=None):
    return _build_sam(
        encoder_embed_dim=1280,
        encoder_depth=32,
        encoder_num_heads=16,
        encoder_global_attn_indexes=[7, 15, 23, 31],
        checkpoint=checkpoint,
    )


build_sam = build_sam_vit_h


def build_sam_vit_l(checkpoint=None):
    return _build_sam(
        encoder_embed_dim=1024,
        encoder_depth=24,
        encoder_num_heads=16,
        encoder_global_attn_indexes=[5, 11, 17, 23],
        checkpoint=checkpoint,
    )


def build_sam_vit_b(checkpoint=None):
    return _build_sam(
        encoder_embed_dim=768,
        encoder_depth=12,
        encoder_num_heads=12,
        encoder_global_attn_indexes=[2, 5, 8, 11],
        checkpoint=checkpoint,
    )


def build_sam_vit_t(checkpoint=None):
    prompt_embed_dim = 256
    image_size = 1024
    vit_patch_size = 16
    image_embedding_size = image_size // vit_patch_size
    mobile_sam = Sam(
            image_encoder=TinyViT(img_size=1024, in_chans=3, num_classes=1000,
                embed_dims=[64, 128, 160, 320],
                depths=[2, 2, 6, 2],
                num_heads=[2, 4, 5, 10],
                window_sizes=[7, 7, 14, 7],
                mlp_ratio=4.,
                drop_rate=0.,
                drop_path_rate=0.0,
                use_checkpoint=False,
                mbconv_expand_ratio=4.0,
                local_conv_size=3,
                layer_lr_decay=0.8
            ),
            prompt_encoder=PromptEncoder(
            embed_dim=prompt_embed_dim,
            image_embedding_size=(image_embedding_size, image_embedding_size),
            input_image_size=(image_size, image_size),
            mask_in_chans=16,
            ),
            mask_decoder=MaskDecoder(
                    num_multimask_outputs=3,
                    transformer=TwoWayTransformer(
                    depth=2,
                    embedding_dim=prompt_embed_dim,
                    mlp_dim=2048,
                    num_heads=8,
                ),
                transformer_dim=prompt_embed_dim,
                iou_head_depth=3,
                iou_head_hidden_dim=256,
            ),
            pixel_mean=[123.675, 116.28, 103.53],
            pixel_std=[58.395, 57.12, 57.375],
        )

    mobile_sam.eval()
    if checkpoint is not None:
        with open(checkpoint, "rb") as f:
            state_dict = torch.load(f)
        mobile_sam.load_state_dict(state_dict)
    return mobile_sam


sam_model_registry = {
    "default": build_sam_vit_h,
    "vit_h": build_sam_vit_h,
    "vit_l": build_sam_vit_l,
    "vit_b": build_sam_vit_b,
    "vit_t": build_sam_vit_t,
}


def _build_sam(
    encoder_embed_dim,
    encoder_depth,
    encoder_num_heads,
    encoder_global_attn_indexes,
    checkpoint=None,
):
    prompt_embed_dim = 256
    image_size = 1024
    vit_patch_size = 16
    image_embedding_size = image_size // vit_patch_size
    sam = Sam(
        image_encoder=ImageEncoderViT(
            depth=encoder_depth,
            embed_dim=encoder_embed_dim,
            img_size=image_size,
            mlp_ratio=4,
            norm_layer=partial(torch.nn.LayerNorm, eps=1e-6),
            num_heads=encoder_num_heads,
            patch_size=vit_patch_size,
            qkv_bias=True,
            use_rel_pos=True,
            global_attn_indexes=encoder_global_attn_indexes,
            window_size=14,
            out_chans=prompt_embed_dim,
        ),
        prompt_encoder=PromptEncoder(
            embed_dim=prompt_embed_dim,
            image_embedding_size=(image_embedding_size, image_embedding_size),
            input_image_size=(image_size, image_size),
            mask_in_chans=16,
        ),
        mask_decoder=MaskDecoder(
            num_multimask_outputs=3,
            transformer=TwoWayTransformer(
                depth=2,
                embedding_dim=prompt_embed_dim,
                mlp_dim=2048,
                num_heads=8,
            ),
            transformer_dim=prompt_embed_dim,
            iou_head_depth=3,
            iou_head_hidden_dim=256,
        ),
        pixel_mean=[123.675, 116.28, 103.53],
        pixel_std=[58.395, 57.12, 57.375],
    )
    sam.eval()
    if checkpoint is not None:
        with open(checkpoint, "rb") as f:
            state_dict = torch.load(f)
        sam.load_state_dict(state_dict)
    return sam





# -*- coding: utf-8 -*-
# @Author  : LG

from PyQt5 import QtWidgets, QtCore, QtGui
from PyQt5.QtCore import QThread, pyqtSignal
from ISAT.ui.auto_segment import Ui_Dialog
from ISAT.configs import CONTOURMode
from xml.etree import ElementTree as ET
from PIL import Image
import numpy as np
from json import dump
import imgviz
import yaml
import cv2
import os

class AutoSegmentThread(QThread):
    message = pyqtSignal(int, int, str)
    def __init__(self, mainwindow):
        super(AutoSegmentThread, self).__init__()
        self.mainwindow = mainwindow
        self.cancel = False
        self.image_dir = None
        self.label_dir = None
        self.save_dir = None

    def run(self):
        image_names = []
        cates = set()
        suffixs = tuple(
            ['{}'.format(fmt.data().decode('ascii').lower()) for fmt in QtGui.QImageReader.supportedImageFormats()])
        for f in os.listdir(self.image_dir):
            if f.lower().endswith(suffixs):
                image_names.append(f)
        image_names.sort()
        images_num = len(image_names)

        for index, image_name in enumerate(image_names):
            self.message.emit(index+1, images_num, '{}'.format(image_name))
            image_path = os.path.join(self.image_dir, image_name)
            if self.cancel:
                self.message.emit(-1, -1, '{}'.format('Cancel!'))
                return
            xml_name = '.'.join(image_name.split('.')[:-1]) + '.xml'
            xml_path = os.path.join(self.label_dir, xml_name)
            if not os.path.exists(xml_path):
                self.message.emit(-1, -1, '{}'.format("Don't exist xml file."))
                continue
            self.message.emit(-1, -1, '{}'.format('Sam encoding...'))
            # sam
            try:
                image_data = np.array(Image.open(image_path))
                self.mainwindow.segany.set_image(image_data)
            except Exception as e:
                self.message.emit(-1, -1, 'Sam error when encoding image: {}'.format(e))
                continue

            # xml
            try:
                tree = ET.parse(xml_path)
                root = tree.getroot()
                objs = root.findall('object')
                size = root.find('size')
                width = size.find('width').text
                height = size.find('height').text
                depth = size.find('depth').text
            except Exception as e:
                self.message.emit(-1, -1, 'Load xml error: {}'.format(e))
                continue

            # isat
            dataset = {}
            dataset['info'] = {}
            dataset['info']['description'] = 'ISAT'
            dataset['info']['folder'] = self.image_dir
            dataset['info']['name'] = image_name
            dataset['info']['width'] = width
            dataset['info']['height'] = height
            dataset['info']['depth'] = depth
            dataset['info']['note'] = ''
            dataset['objects'] = []

            for group, obj in enumerate(objs):
                name = obj.find('name').text
                difficult = obj.find('difficult').text
                bndbox = obj.find('bndbox')
                xmin = float(bndbox.find('xmin').text)
                ymin = float(bndbox.find('ymin').text)
                xmax = float(bndbox.find('xmax').text)
                ymax = float(bndbox.find('ymax').text)

                cates.add(name)

                masks = self.mainwindow.segany.predict_with_box_prompt(box=np.array([xmin, ymin, xmax, ymax]))

                masks = masks.astype('uint8') * 255
                h, w = masks.shape[-2:]
                masks = masks.reshape(h, w)

                if self.mainwindow.scene.contour_mode == CONTOURMode.SAVE_ALL:
                    # 当保留所有轮廓时，检测所有轮廓，并建立二层等级关系
                    contours, hierarchy = cv2.findContours(masks, cv2.RETR_CCOMP, cv2.CHAIN_APPROX_TC89_KCOS)
                else:
                    # 当只保留外轮廓或单个mask时，只检测外轮廓
                    contours, hierarchy = cv2.findContours(masks, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_TC89_KCOS)

                if self.mainwindow.scene.contour_mode == CONTOURMode.SAVE_MAX_ONLY:
                    largest_contour = max(contours, key=cv2.contourArea)  # 只保留面积最大的轮廓
                    contours = [largest_contour]

                for _, contour in enumerate(contours):
                    # polydp
                    if self.mainwindow.cfg['software']['use_polydp']:
                        epsilon_factor = 0.001
                        epsilon = epsilon_factor * cv2.arcLength(contour, True)
                        contour = cv2.approxPolyDP(contour, epsilon, True)

                    object = {}
                    object['category'] = name
                    object['group'] = group + 1
                    object['segmentation'] = [(int(point[0][0]), int(point[0][1])) for point in contour]
                    object['area'] = None
                    object['layer'] = group + 1
                    object['bbox'] = [xmin, ymin, xmax, ymax]
                    object['iscrowd'] = 0
                    object['note'] = ''
                    dataset['objects'].append(object)

            try:
                save_path = os.path.join(self.save_dir, '.'.join(image_name.split('.')[:-1]) + '.json')
                with open(save_path, 'w') as f:
                    dump(dataset, f, indent=4)
                self.message.emit(-1, -1, '{}'.format('Save finished!'))
            except Exception as e:
                self.message.emit(-1, -1, 'Save ISAT json error: {}'.format(e))

        # 类别文件
        try:
            cmap = imgviz.label_colormap()
            cates = list(cates)
            cates = sorted(cates)
            categories = []
            for index, cat in enumerate(cates):
                r, g, b = cmap[index + 1]
                categories.append({
                    'name': cat if isinstance(cat, str) else str(cat),
                    'color': "#{:02x}{:02x}{:02x}".format(r, g, b)
                })
            s = yaml.dump({'label': categories})
            with open(os.path.join(self.save_dir, 'isat.yaml'), 'w') as f:
                f.write(s)
            self.message.emit(-1, -1, 'Save ISAT yaml finished!')

        except Exception as e:
            self.message.emit(-1, -1, 'Save ISAT yaml error: {}'.format(e))


class AutoSegmentDialog(QtWidgets.QDialog, Ui_Dialog):
    def __init__(self, parent, mainwindow):
        super(AutoSegmentDialog, self).__init__(parent)
        self.setupUi(self)
        self.mainwindow = mainwindow
        self.image_dir = None
        self.label_dir = None
        self.save_dir = None

        self.auto_segment_thread = AutoSegmentThread(self.mainwindow)
        self.auto_segment_thread.message.connect(self.print_message)

        self.setWindowModality(QtCore.Qt.WindowModality.WindowModal)
        self.init_connect()

    def open_dir(self):
        dir = QtWidgets.QFileDialog.getExistingDirectory(self, caption='Open dir')
        if self.sender() == self.pushButton_image_dir:
            lineEdit = self.lineEdit_image_dir
        elif self.sender() == self.pushButton_label_dir:
            lineEdit = self.lineEdit_label_dir
        elif self.sender() == self.pushButton_save_dir:
            lineEdit = self.lineEdit_save_dir
        else:
            return
        if dir:
            if lineEdit is not None:
                lineEdit.setText(dir)
        else:
            if lineEdit is not None:
                lineEdit.clear()

    def start(self):
        self.auto_segment_thread.cancel = False

        self.image_dir = self.lineEdit_image_dir.text()
        self.label_dir = self.lineEdit_label_dir.text()
        self.save_dir = self.lineEdit_save_dir.text()

        if self.image_dir == '' or self.label_dir == '' or self.save_dir == '':
            self.textBrowser.append('不能为空')
            return

        self.auto_segment_thread.image_dir = self.image_dir
        self.auto_segment_thread.label_dir = self.label_dir
        self.auto_segment_thread.save_dir = self.save_dir
        self.auto_segment_thread.start()

    def cancel(self):
        self.auto_segment_thread.cancel = True

    def print_message(self, index, all, message):
        if index > 0:
            self.progressBar.setValue(index)
        if all > 0:
            self.progressBar.setMaximum(all)
        if message:
            self.textBrowser.append('{} | {}'.format(
                '{:>8s}/{:<8s}'.format(str(index), str(all)) if (index > 0 and all > 0) else '{:>8s} {:<8s}'.format('', ''), message))
            print('{} | {}'.format(
                '{:>8s}/{:<8s}'.format(str(index), str(all)) if (index > 0 and all > 0) else '{:>8s} {:<8s}'.format('', ''), message))

    def init_connect(self):
        self.pushButton_image_dir.clicked.connect(self.open_dir)
        self.pushButton_label_dir.clicked.connect(self.open_dir)
        self.pushButton_save_dir.clicked.connect(self.open_dir)
        self.pushButton_start.clicked.connect(self.start)
        self.pushButton_cancel.clicked.connect(self.cancel)



# -*- coding: utf-8 -*-
# @Author  : LG
# from https://blog.51cto.com/u_15872074/5841477

from PyQt5.QtWidgets import QWidget
from PyQt5.QtCore import pyqtSignal, QTimer, QRect, QRectF, Qt
from PyQt5.QtGui import QColor, QFont, QPainter, QPainterPath, QPen


class SwitchBtn(QWidget):
    #信号
    checkedChanged = pyqtSignal(bool)
    def __init__(self,parent=None):
        super(QWidget, self).__init__(parent)

        self.checked = False
        self.bgColorOff = QColor('#6E798A')
        self.bgColorOn = QColor('#70AEFF')

        self.sliderColorOff = QColor(255, 255, 255)
        self.sliderColorOn = QColor(255, 255, 255)

        self.textColorOff = QColor(255, 255, 255)
        self.textColorOn = QColor(255, 255, 255)

        self.textOff = "OFF"
        self.textOn = "ON"

        self.space = 2
        self.rectRadius = 5

        self.step = self.width() / 50
        self.startX = 0
        self.endX = 0

        self.timer = QTimer(self)  # 初始化一个定时器
        self.timer.timeout.connect(self.updateValue)  # 计时结束调用operate()方法
        self.setFont(QFont("timesnewroman", 10))

    def updateValue(self):
        if self.checked:
            if self.startX < self.endX:
                self.startX = self.startX + self.step
            else:
                self.startX = self.endX
                self.timer.stop()
        else:
            if self.startX  > self.endX:
                self.startX = self.startX - self.step
            else:
                self.startX = self.endX
                self.timer.stop()
        self.update()

    def mousePressEvent(self,event):
        self.checked = not self.checked
        #发射信号
        self.checkedChanged.emit(self.checked)
        # 每次移动的步长为宽度的50分之一
        self.step = self.width() / 50
        #状态切换改变后自动计算终点坐标
        if self.checked:
            self.endX = self.width() - self.height()
        else:
            self.endX = 0
        self.timer.start(5)

    def paintEvent(self, evt):
        #绘制准备工作, 启用反锯齿
            painter = QPainter()
            painter.begin(self)
            painter.setRenderHint(QPainter.Antialiasing)
            #绘制背景
            self.drawBg(evt, painter)
            #绘制滑块
            self.drawSlider(evt, painter)
            #绘制文字
            self.drawText(evt, painter)
            painter.end()

    def drawText(self, event, painter):
        painter.save()
        if self.checked:
            painter.setPen(self.textColorOn)
            painter.drawText(0, 0, int(self.width() / 2 + self.space * 2), int(self.height()), Qt.AlignCenter, self.textOn)
        else:
            painter.setPen(self.textColorOff)
            painter.drawText(int(self.width() / 2), 0, int(self.width() / 2 - self.space), int(self.height()), Qt.AlignCenter, self.textOff)
        painter.restore()

    def drawBg(self, event, painter):
        painter.save()
        painter.setPen(Qt.NoPen)
        if self.checked:
            painter.setBrush(self.bgColorOn)
        else:
            painter.setBrush(self.bgColorOff)
        #半径为高度的一半
        radius = self.height() / 2
        #圆的宽度为高度
        circleWidth = self.height()

        path = QPainterPath()
        path.moveTo(radius, 0)
        path.arcTo(QRectF(0, 0, circleWidth, circleWidth), 90, 180)
        path.lineTo(self.width() - radius, self.height())
        path.arcTo(QRectF(self.width() - self.height(), 0, circleWidth, circleWidth), 270, 180)
        path.lineTo(radius, 0)

        painter.drawPath(path)
        painter.restore()

    def drawSlider(self, event, painter):
        painter.save()
        if self.checked:
            painter.setBrush(self.sliderColorOn)
            painter.setPen(QPen(self.sliderColorOn, 1))
        else:
            painter.setBrush(self.sliderColorOff)

        sliderWidth = self.height() - self.space * 2
        sliderRect = QRectF(self.startX + self.space, self.space, sliderWidth, sliderWidth)
        painter.drawEllipse(sliderRect)

        painter.restore()

    def setChecked(self, checked=False):
        if self.checked != checked:
            self.mousePressEvent(None)



# -*- coding: utf-8 -*-
# @Author  : LG

from PyQt5 import QtCore, QtWidgets, QtGui
from ISAT.annotation import Object
import typing
from ISAT.configs import STATUSMode, CLICKMode, DRAWMode, CONTOURMode


class PromptPoint(QtWidgets.QGraphicsPathItem):
    def __init__(self, pos, type=0):
        super(PromptPoint, self).__init__()
        self.color = QtGui.QColor('#0000FF') if type==0 else QtGui.QColor('#00FF00')
        self.color.setAlpha(255)
        self.painterpath = QtGui.QPainterPath()
        self.painterpath.addEllipse(
            QtCore.QRectF(-1, -1, 2, 2))
        self.setPath(self.painterpath)
        self.setBrush(self.color)
        self.setPen(QtGui.QPen(self.color, 3))
        self.setZValue(1e5)

        self.setPos(pos)


class Vertex(QtWidgets.QGraphicsPathItem):
    def __init__(self, polygon, color, nohover_size=2):
        super(Vertex, self).__init__()
        self.polygon = polygon
        self.color = color
        self.color.setAlpha(255)
        self.nohover_size = nohover_size
        self.hover_size = self.nohover_size + 2
        self.line_width = 0

        self.nohover = QtGui.QPainterPath()
        self.nohover.addEllipse(QtCore.QRectF(-self.nohover_size//2, -self.nohover_size//2, self.nohover_size, self.nohover_size))
        self.hover = QtGui.QPainterPath()
        self.hover.addRect(QtCore.QRectF(-self.nohover_size//2, -self.nohover_size//2, self.nohover_size, self.nohover_size))

        self.setPath(self.nohover)
        self.setBrush(self.color)
        self.setPen(QtGui.QPen(self.color, self.line_width))
        self.setFlag(QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, True)
        self.setFlag(QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemIsMovable, True)
        self.setFlag(QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemSendsGeometryChanges, True)
        self.setAcceptHoverEvents(True)
        self.setZValue(1e5)

    def setColor(self, color):
        self.color = QtGui.QColor(color)
        self.color.setAlpha(255)
        self.setPen(QtGui.QPen(self.color, self.line_width))
        self.setBrush(self.color)

    def itemChange(self, change: 'QtWidgets.QGraphicsItem.GraphicsItemChange', value: typing.Any):
        if change == QtWidgets.QGraphicsItem.GraphicsItemChange.ItemSelectedHasChanged:
            self.scene().mainwindow.actionDelete.setEnabled(self.isSelected())
            if self.isSelected():
                selected_color = QtGui.QColor('#00A0FF')
                self.setBrush(selected_color)
            else:
                self.setBrush(self.color)

        if change == QtWidgets.QGraphicsItem.GraphicsItemChange.ItemPositionChange and self.isEnabled():
            # 限制顶点移动到图外
            if value.x() < 0:
                value.setX(0)
            if value.x() > self.scene().width()-1:
                value.setX(self.scene().width()-1)
            if value.y() < 0:
                value.setY(0)
            if value.y() > self.scene().height()-1:
                value.setY(self.scene().height()-1)
            index = self.polygon.vertexs.index(self)
            self.polygon.movePoint(index, value)

        return super(Vertex, self).itemChange(change, value)
    
    def hoverEnterEvent(self, event: 'QGraphicsSceneHoverEvent'):
        self.scene().hovered_vertex = self
        if self.scene().mode == STATUSMode.CREATE: # CREATE
            self.setCursor(QtGui.QCursor(QtCore.Qt.CursorShape.CrossCursor))
        else: # EDIT, VIEW
            self.setCursor(QtGui.QCursor(QtCore.Qt.CursorShape.OpenHandCursor))
            if not self.isSelected():
                self.setBrush(QtGui.QColor(255, 255, 255, 255))
            self.setPath(self.hover)
        super(Vertex, self).hoverEnterEvent(event)

    def hoverLeaveEvent(self, event: 'QGraphicsSceneHoverEvent'):
        self.scene().hovered_vertex = None
        if not self.isSelected():
            self.setBrush(self.color)
        self.setPath(self.nohover)
        super(Vertex, self).hoverLeaveEvent(event)


class Polygon(QtWidgets.QGraphicsPolygonItem):
    def __init__(self):
        super(Polygon, self).__init__(parent=None)
        self.line_width = 1
        self.hover_alpha = 150
        self.nohover_alpha = 80
        self.points = []
        self.vertexs = []
        self.category = ''
        self.group = 0
        self.iscrowd = 0
        self.note = ''

        self.rxmin, self.rxmax, self.rymin, self.rymax = 0, 0, 0, 0 # 用于绘画完成后，记录多边形的各边界，此处与points对应
        self.color = QtGui.QColor('#ff0000')
        self.is_drawing = True

        self.setPen(QtGui.QPen(self.color, self.line_width))
        self.setBrush(QtGui.QBrush(self.color, QtCore.Qt.BrushStyle.FDiagPattern))

        self.setAcceptHoverEvents(True)
        self.setFlag(QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, True)
        self.setFlag(QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemIsMovable, True)
        self.setFlag(QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemSendsGeometryChanges, True)
        self.setZValue(1e5)

    def addPoint(self, point):
        self.points.append(point)
        vertex = Vertex(self, self.color, self.scene().mainwindow.cfg['software']['vertex_size'] * 2)
        # 添加路径点
        self.scene().addItem(vertex)
        self.vertexs.append(vertex)
        vertex.setPos(point)

    def movePoint(self, index, point):
        if not 0 <= index < len(self.points):
            return
        self.points[index] = self.mapFromScene(point)

        self.redraw()
        if self.scene().mainwindow.load_finished and not self.is_drawing:
            self.scene().mainwindow.set_saved_state(False)

    def removePoint(self, index):
        if not self.points:
            return
        self.points.pop(index)
        vertex = self.vertexs.pop(index)
        self.scene().removeItem(vertex)
        del vertex
        self.redraw()

    def delete(self):
        self.points.clear()
        while self.vertexs:
            vertex = self.vertexs.pop()
            self.scene().removeItem(vertex)
            del vertex

    def moveVertex(self, index, point):
        if not 0 <= index < len(self.vertexs):
            return
        vertex = self.vertexs[index]
        vertex.setEnabled(False)
        vertex.setPos(point)
        vertex.setEnabled(True)

    def itemChange(self, change: 'QGraphicsItem.GraphicsItemChange', value: typing.Any):
        if (change == QtWidgets.QGraphicsItem.GraphicsItemChange.ItemSelectedHasChanged
                and not self.is_drawing
                and self.scene().mode!=STATUSMode.CREATE): # 选中改变
            if self.isSelected():
                color = QtGui.QColor('#00A0FF')
                color.setAlpha(self.hover_alpha)
                self.setBrush(color)
                self.scene().selected_polygons_list.append(self)
            else:
                self.color.setAlpha(self.nohover_alpha)
                self.setBrush(self.color)
                if self in self.scene().selected_polygons_list:
                    self.scene().selected_polygons_list.remove(self)
            self.scene().mainwindow.annos_dock_widget.set_selected(self) # 更新label面板

        if change == QtWidgets.QGraphicsItem.GraphicsItemChange.ItemPositionHasChanged:  # ItemPositionHasChanged
            bias = value
            l, t, b, r = self.boundingRect().left(), self.boundingRect().top(), self.boundingRect().bottom(), self.boundingRect().right()
            if l + bias.x() < 0: bias.setX(-l)
            if r + bias.x() > self.scene().width(): bias.setX(self.scene().width()-r)
            if t + bias.y() < 0: bias.setY(-t)
            if b + bias.y() > self.scene().height(): bias.setY(self.scene().height()-b)

            for index, point in enumerate(self.points):
                self.moveVertex(index, point+bias)

            if self.scene().mainwindow.load_finished and not self.is_drawing:
                self.scene().mainwindow.set_saved_state(False)

        if change == QtWidgets.QGraphicsItem.GraphicsItemChange.ItemPositionChange:
            value = 0 if self.is_drawing else value
        if change == QtWidgets.QGraphicsItem.GraphicsItemChange.ItemSelectedHasChanged and self.isSelected():
            self.setSelected(not self.is_drawing)
        return super(Polygon, self).itemChange(change, value)

    def hoverEnterEvent(self, event: 'QGraphicsSceneHoverEvent'):
        if not self.is_drawing and not self.isSelected():
            self.color.setAlpha(self.hover_alpha)
            self.setBrush(self.color)
        super(Polygon, self).hoverEnterEvent(event)

    def hoverLeaveEvent(self, event: 'QGraphicsSceneHoverEvent'):
        if not self.is_drawing and not self.isSelected():
            self.color.setAlpha(self.nohover_alpha)
            self.setBrush(self.color)
        super(Polygon, self).hoverEnterEvent(event)

    def mouseDoubleClickEvent(self, event: 'QGraphicsSceneMouseEvent'):
        if event.button() == QtCore.Qt.MouseButton.LeftButton:
            self.scene().mainwindow.category_edit_widget.polygon = self
            self.scene().mainwindow.category_edit_widget.load_cfg()
            self.scene().mainwindow.category_edit_widget.show()

    def redraw(self):
        if len(self.points) < 1:
            return
        xs = [p.x() for p in self.points]
        ys = [p.y() for p in self.points]
        self.rxmin, self.rymin, self.rxmax, self.rymax = min(xs), min(ys), max(xs), max(ys)
        self.setPolygon(QtGui.QPolygonF(self.points))

    def change_color(self, color):
        self.color = color
        if not self.scene().mainwindow.cfg['software']['show_edge']:
            color.setAlpha(0)
        self.setPen(QtGui.QPen(color, self.line_width))
        self.color.setAlpha(self.nohover_alpha)
        self.setBrush(self.color)
        for vertex in self.vertexs:
            vertex_color = self.color
            vertex_color.setAlpha(255)
            vertex.setPen(QtGui.QPen(vertex_color, self.line_width))
            vertex.setBrush(vertex_color)

    def set_drawed(self, category, group, iscrowd, note, color:QtGui.QColor, layer=None):
        self.is_drawing = False
        self.category = category
        if isinstance(group, str):
            group = 0 if group == '' else int(group)
        self.group = group
        self.iscrowd = iscrowd
        self.note = note

        self.color = color
        if not self.scene().mainwindow.cfg['software']['show_edge']:
            color.setAlpha(0)
        self.setPen(QtGui.QPen(color, self.line_width))
        self.color.setAlpha(self.nohover_alpha)
        self.setBrush(self.color)
        if layer is not None:
            self.setZValue(layer)
            for vertex in self.vertexs:
                vertex.setZValue(layer)
        for vertex in self.vertexs:
            vertex.setColor(color)

    def calculate_area(self):
        area = 0
        num_points = len(self.points)
        for i in range(num_points):
            p1 = self.points[i]
            p2 = self.points[(i + 1) % num_points]
            d = p1.x() * p2.y() - p2.x() * p1.y()
            area += d
        return abs(area) / 2

    def load_object(self, object):
        segmentation = object.segmentation
        for x, y in segmentation:
            point = QtCore.QPointF(x, y)
            self.addPoint(point)
        color = self.scene().mainwindow.category_color_dict.get(object.category, '#000000')
        self.set_drawed(object.category, object.group, object.iscrowd, object.note, QtGui.QColor(color), object.layer)  # ...

    def to_object(self):
        if self.is_drawing:
            return None
        segmentation = []
        for point in self.points:
            point = point + self.pos()
            segmentation.append((round(point.x(), 2), round(point.y(), 2)))
        xmin = self.boundingRect().x() + self.pos().x()
        ymin = self.boundingRect().y() + self.pos().y()
        xmax = xmin + self.boundingRect().width()
        ymax = ymin + self.boundingRect().height()

        object = Object(self.category, group=self.group, segmentation=segmentation,
                        area=self.calculate_area(), layer=self.zValue(), bbox=(xmin, ymin, xmax, ymax), iscrowd=self.iscrowd, note=self.note)
        return object


class LineVertex(QtWidgets.QGraphicsPathItem):
    def __init__(self, polygon, color, nohover_size=2):
        super(LineVertex, self).__init__()
        self.polygon = polygon
        self.color = color
        self.color.setAlpha(255)
        self.nohover_size = nohover_size
        self.hover_size = self.nohover_size + 2
        self.line_width = 0

        self.nohover = QtGui.QPainterPath()
        self.nohover.addEllipse(QtCore.QRectF(-self.nohover_size//2, -self.nohover_size//2, self.nohover_size, self.nohover_size))
        self.hover = QtGui.QPainterPath()
        self.hover.addRect(QtCore.QRectF(-self.nohover_size//2, -self.nohover_size//2, self.nohover_size, self.nohover_size))

        self.setPath(self.nohover)
        self.setBrush(self.color)
        self.setPen(QtGui.QPen(self.color, self.line_width))
        self.setFlag(QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, False)
        self.setFlag(QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemIsMovable, True)
        self.setFlag(QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemSendsGeometryChanges, True)
        self.setAcceptHoverEvents(True)
        self.setZValue(1e5)

    def itemChange(self, change: 'QtWidgets.QGraphicsItem.GraphicsItemChange', value: typing.Any):
        if change == QtWidgets.QGraphicsItem.GraphicsItemChange.ItemSelectedHasChanged:
            self.scene().mainwindow.actionDelete.setEnabled(self.isSelected())
            if self.isSelected():
                selected_color = QtGui.QColor('#00A0FF')
                self.setBrush(selected_color)
            else:
                self.setBrush(self.color)

        if change == QtWidgets.QGraphicsItem.GraphicsItemChange.ItemPositionChange and self.isEnabled():
            # 限制顶点移动到图外
            if value.x() < 0:
                value.setX(0)
            if value.x() > self.scene().width()-1:
                value.setX(self.scene().width()-1)
            if value.y() < 0:
                value.setY(0)
            if value.y() > self.scene().height()-1:
                value.setY(self.scene().height()-1)
            index = self.polygon.vertexs.index(self)
            self.polygon.movePoint(index, value)

        return super(LineVertex, self).itemChange(change, value)

class Line(QtWidgets.QGraphicsPathItem):
    def __init__(self):
        super().__init__(parent=None)
        self.line_width = 1
        # self.hover_alpha = 150
        # self.nohover_alpha = 80
        self.points = []
        self.vertexs = []
        self.color = QtGui.QColor('#ff0000')

    def addPoint(self, point):
        self.points.append(point)
        vertex = LineVertex(self, self.color, self.scene().mainwindow.cfg['software']['vertex_size'] * 2)
        # 添加路径点
        self.scene().addItem(vertex)
        self.vertexs.append(vertex)
        vertex.setPos(point)

    def movePoint(self, index, point):
        if not 0 <= index < len(self.points):
            return
        self.points[index] = self.mapFromScene(point)
        self.redraw()

    def removePoint(self, index):
        if not self.points:
            return
        self.points.pop(index)
        vertex = self.vertexs.pop(index)
        self.scene().removeItem(vertex)
        del vertex
        self.redraw()

    def delete(self):
        self.points.clear()
        while self.vertexs:
            vertex = self.vertexs.pop()
            self.scene().removeItem(vertex)
            del vertex

    def redraw(self):
        if len(self.points) < 1:
            return
        xs = [p.x() for p in self.points]
        ys = [p.y() for p in self.points]
        self.rxmin, self.rymin, self.rxmax, self.rymax = min(xs), min(ys), max(xs), max(ys)

        line_path = QtGui.QPainterPath()
        if self.points:
            line_path.moveTo(self.points[0])
            for point in self.points[1:]:
                line_path.lineTo(point)

        self.setPath(line_path)


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/lg/PycharmProjects/ISAT_with_segment_anything/ISAT/ui/category_dock.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(294, 462)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.lineEdit_search_category = QtWidgets.QLineEdit(Form)
        self.lineEdit_search_category.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.lineEdit_search_category.setObjectName("lineEdit_search_category")
        self.verticalLayout.addWidget(self.lineEdit_search_category)
        self.listWidget = QtWidgets.QListWidget(Form)
        self.listWidget.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.listWidget.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.listWidget.setObjectName("listWidget")
        self.verticalLayout.addWidget(self.listWidget)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(0, 0, -1, 0)
        self.horizontalLayout.setSpacing(6)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.pushButton_group_mode = QtWidgets.QPushButton(Form)
        self.pushButton_group_mode.setMaximumSize(QtCore.QSize(60, 16777215))
        self.pushButton_group_mode.setObjectName("pushButton_group_mode")
        self.horizontalLayout.addWidget(self.pushButton_group_mode)
        self.pushButton_decrease = QtWidgets.QPushButton(Form)
        self.pushButton_decrease.setMaximumSize(QtCore.QSize(20, 16777215))
        self.pushButton_decrease.setObjectName("pushButton_decrease")
        self.horizontalLayout.addWidget(self.pushButton_decrease)
        self.lineEdit_currentGroup = QtWidgets.QLineEdit(Form)
        self.lineEdit_currentGroup.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.lineEdit_currentGroup.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.lineEdit_currentGroup.setText("")
        self.lineEdit_currentGroup.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lineEdit_currentGroup.setObjectName("lineEdit_currentGroup")
        self.horizontalLayout.addWidget(self.lineEdit_currentGroup)
        self.pushButton_increase = QtWidgets.QPushButton(Form)
        self.pushButton_increase.setMaximumSize(QtCore.QSize(20, 16777215))
        self.pushButton_increase.setObjectName("pushButton_increase")
        self.horizontalLayout.addWidget(self.pushButton_increase)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.lineEdit_search_category.setPlaceholderText(_translate("Form", "Search categories."))
        self.pushButton_group_mode.setStatusTip(_translate("Form", "Group mode"))
        self.pushButton_group_mode.setWhatsThis(_translate("Form", "Group mode"))
        self.pushButton_group_mode.setText(_translate("Form", "Auto"))
        self.pushButton_decrease.setStatusTip(_translate("Form", "Current group -"))
        self.pushButton_decrease.setWhatsThis(_translate("Form", "Current group -"))
        self.pushButton_decrease.setText(_translate("Form", "-"))
        self.lineEdit_currentGroup.setStatusTip(_translate("Form", "Current group"))
        self.lineEdit_currentGroup.setWhatsThis(_translate("Form", "Current group"))
        self.pushButton_increase.setStatusTip(_translate("Form", "Current group +"))
        self.pushButton_increase.setWhatsThis(_translate("Form", "Current group +"))
        self.pushButton_increase.setText(_translate("Form", "+"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'Converter_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(1000, 680)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        Dialog.setFont(font)
        self.verticalLayout_13 = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.tabWidget = QtWidgets.QTabWidget(Dialog)
        self.tabWidget.setObjectName("tabWidget")
        self.tab_COCO = QtWidgets.QWidget()
        self.tab_COCO.setObjectName("tab_COCO")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tab_COCO)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.widget = QtWidgets.QWidget(self.tab_COCO)
        self.widget.setObjectName("widget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.widget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.toolBox_coco = QtWidgets.QToolBox(self.widget)
        self.toolBox_coco.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.toolBox_coco.setFrameShadow(QtWidgets.QFrame.Plain)
        self.toolBox_coco.setObjectName("toolBox_coco")
        self.toolbox_item_coco2isat = QtWidgets.QWidget()
        self.toolbox_item_coco2isat.setGeometry(QtCore.QRect(0, 0, 138, 109))
        self.toolbox_item_coco2isat.setObjectName("toolbox_item_coco2isat")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.toolbox_item_coco2isat)
        self.verticalLayout_2.setContentsMargins(9, 9, 9, 9)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.widget_3 = QtWidgets.QWidget(self.toolbox_item_coco2isat)
        self.widget_3.setObjectName("widget_3")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.widget_3)
        self.verticalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.widget_7 = QtWidgets.QWidget(self.widget_3)
        self.widget_7.setObjectName("widget_7")
        self.gridLayout = QtWidgets.QGridLayout(self.widget_7)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.pushButton_coco2isat_coco_json_path = QtWidgets.QPushButton(self.widget_7)
        self.pushButton_coco2isat_coco_json_path.setObjectName("pushButton_coco2isat_coco_json_path")
        self.gridLayout.addWidget(self.pushButton_coco2isat_coco_json_path, 0, 1, 1, 1)
        self.lineEdit_coco2isat_isat_json_root = QtWidgets.QLineEdit(self.widget_7)
        self.lineEdit_coco2isat_isat_json_root.setReadOnly(True)
        self.lineEdit_coco2isat_isat_json_root.setObjectName("lineEdit_coco2isat_isat_json_root")
        self.gridLayout.addWidget(self.lineEdit_coco2isat_isat_json_root, 1, 0, 1, 1)
        self.lineEdit_coco2isat_coco_json_path = QtWidgets.QLineEdit(self.widget_7)
        self.lineEdit_coco2isat_coco_json_path.setReadOnly(True)
        self.lineEdit_coco2isat_coco_json_path.setObjectName("lineEdit_coco2isat_coco_json_path")
        self.gridLayout.addWidget(self.lineEdit_coco2isat_coco_json_path, 0, 0, 1, 1)
        self.pushButton_coco2isat_isat_json_root = QtWidgets.QPushButton(self.widget_7)
        self.pushButton_coco2isat_isat_json_root.setObjectName("pushButton_coco2isat_isat_json_root")
        self.gridLayout.addWidget(self.pushButton_coco2isat_isat_json_root, 1, 1, 1, 1)
        self.verticalLayout_7.addWidget(self.widget_7)
        self.widget_8 = QtWidgets.QWidget(self.widget_3)
        self.widget_8.setObjectName("widget_8")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.widget_8)
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem = QtWidgets.QSpacerItem(445, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.checkBox_coco2isat_keep_crowd = QtWidgets.QCheckBox(self.widget_8)
        self.checkBox_coco2isat_keep_crowd.setObjectName("checkBox_coco2isat_keep_crowd")
        self.horizontalLayout_3.addWidget(self.checkBox_coco2isat_keep_crowd)
        self.verticalLayout_7.addWidget(self.widget_8)
        self.verticalLayout_2.addWidget(self.widget_3)
        self.toolBox_coco.addItem(self.toolbox_item_coco2isat, "")
        self.toolbox_item_isat2coco = QtWidgets.QWidget()
        self.toolbox_item_isat2coco.setGeometry(QtCore.QRect(0, 0, 960, 153))
        self.toolbox_item_isat2coco.setObjectName("toolbox_item_isat2coco")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.toolbox_item_isat2coco)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.widget_4 = QtWidgets.QWidget(self.toolbox_item_isat2coco)
        self.widget_4.setObjectName("widget_4")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.widget_4)
        self.verticalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.widget_5 = QtWidgets.QWidget(self.widget_4)
        self.widget_5.setObjectName("widget_5")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.widget_5)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.lineEdit_isat2coco_isat_json_root = QtWidgets.QLineEdit(self.widget_5)
        self.lineEdit_isat2coco_isat_json_root.setReadOnly(True)
        self.lineEdit_isat2coco_isat_json_root.setObjectName("lineEdit_isat2coco_isat_json_root")
        self.gridLayout_2.addWidget(self.lineEdit_isat2coco_isat_json_root, 0, 0, 1, 1)
        self.pushButton_isat2coco_isat_json_root = QtWidgets.QPushButton(self.widget_5)
        self.pushButton_isat2coco_isat_json_root.setObjectName("pushButton_isat2coco_isat_json_root")
        self.gridLayout_2.addWidget(self.pushButton_isat2coco_isat_json_root, 0, 1, 1, 1)
        self.lineEdit_isat2coco_coco_json_path = QtWidgets.QLineEdit(self.widget_5)
        self.lineEdit_isat2coco_coco_json_path.setReadOnly(True)
        self.lineEdit_isat2coco_coco_json_path.setObjectName("lineEdit_isat2coco_coco_json_path")
        self.gridLayout_2.addWidget(self.lineEdit_isat2coco_coco_json_path, 1, 0, 1, 1)
        self.pushButton_isat2coco_coco_json_path = QtWidgets.QPushButton(self.widget_5)
        self.pushButton_isat2coco_coco_json_path.setObjectName("pushButton_isat2coco_coco_json_path")
        self.gridLayout_2.addWidget(self.pushButton_isat2coco_coco_json_path, 1, 1, 1, 1)
        self.verticalLayout_6.addWidget(self.widget_5)
        self.widget_6 = QtWidgets.QWidget(self.widget_4)
        self.widget_6.setObjectName("widget_6")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.widget_6)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem1 = QtWidgets.QSpacerItem(445, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.checkBox_isat2coco_keep_crowd = QtWidgets.QCheckBox(self.widget_6)
        self.checkBox_isat2coco_keep_crowd.setObjectName("checkBox_isat2coco_keep_crowd")
        self.horizontalLayout_2.addWidget(self.checkBox_isat2coco_keep_crowd)
        self.verticalLayout_6.addWidget(self.widget_6)
        self.verticalLayout_5.addWidget(self.widget_4)
        self.toolBox_coco.addItem(self.toolbox_item_isat2coco, "")
        self.verticalLayout.addWidget(self.toolBox_coco)
        self.label_2 = QtWidgets.QLabel(self.widget)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.label_2.setFont(font)
        self.label_2.setStyleSheet("color: rgb(255, 0, 0);")
        self.label_2.setObjectName("label_2")
        self.verticalLayout.addWidget(self.label_2)
        self.verticalLayout_4.addWidget(self.widget)
        self.tabWidget.addTab(self.tab_COCO, "")
        self.tab_YOLO = QtWidgets.QWidget()
        self.tab_YOLO.setObjectName("tab_YOLO")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.tab_YOLO)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.widget_9 = QtWidgets.QWidget(self.tab_YOLO)
        self.widget_9.setObjectName("widget_9")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.widget_9)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.toolBox_yolo = QtWidgets.QToolBox(self.widget_9)
        self.toolBox_yolo.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.toolBox_yolo.setFrameShadow(QtWidgets.QFrame.Plain)
        self.toolBox_yolo.setObjectName("toolBox_yolo")
        self.toolbox_item_yolo2isat = QtWidgets.QWidget()
        self.toolbox_item_yolo2isat.setGeometry(QtCore.QRect(0, 0, 161, 144))
        self.toolbox_item_yolo2isat.setObjectName("toolbox_item_yolo2isat")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.toolbox_item_yolo2isat)
        self.verticalLayout_9.setContentsMargins(9, 9, 9, 9)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.widget_10 = QtWidgets.QWidget(self.toolbox_item_yolo2isat)
        self.widget_10.setObjectName("widget_10")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.widget_10)
        self.verticalLayout_10.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.widget_11 = QtWidgets.QWidget(self.widget_10)
        self.widget_11.setObjectName("widget_11")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.widget_11)
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.lineEdit_yolo2isat_isat_json_root = QtWidgets.QLineEdit(self.widget_11)
        self.lineEdit_yolo2isat_isat_json_root.setReadOnly(True)
        self.lineEdit_yolo2isat_isat_json_root.setObjectName("lineEdit_yolo2isat_isat_json_root")
        self.gridLayout_3.addWidget(self.lineEdit_yolo2isat_isat_json_root, 3, 0, 1, 1)
        self.pushButton_yolo2isat_isat_json_root = QtWidgets.QPushButton(self.widget_11)
        self.pushButton_yolo2isat_isat_json_root.setObjectName("pushButton_yolo2isat_isat_json_root")
        self.gridLayout_3.addWidget(self.pushButton_yolo2isat_isat_json_root, 3, 2, 1, 1)
        self.lineEdit_yolo2isat_yolo_txt_root = QtWidgets.QLineEdit(self.widget_11)
        self.lineEdit_yolo2isat_yolo_txt_root.setReadOnly(True)
        self.lineEdit_yolo2isat_yolo_txt_root.setObjectName("lineEdit_yolo2isat_yolo_txt_root")
        self.gridLayout_3.addWidget(self.lineEdit_yolo2isat_yolo_txt_root, 1, 0, 1, 1)
        self.pushButton_yolo2isat_yolo_txt_root = QtWidgets.QPushButton(self.widget_11)
        self.pushButton_yolo2isat_yolo_txt_root.setObjectName("pushButton_yolo2isat_yolo_txt_root")
        self.gridLayout_3.addWidget(self.pushButton_yolo2isat_yolo_txt_root, 1, 2, 1, 1)
        self.lineEdit_yolo2isat_yolo_image_root = QtWidgets.QLineEdit(self.widget_11)
        self.lineEdit_yolo2isat_yolo_image_root.setReadOnly(True)
        self.lineEdit_yolo2isat_yolo_image_root.setObjectName("lineEdit_yolo2isat_yolo_image_root")
        self.gridLayout_3.addWidget(self.lineEdit_yolo2isat_yolo_image_root, 0, 0, 1, 1)
        self.pushButton_yolo2isat_yolo_image_root = QtWidgets.QPushButton(self.widget_11)
        self.pushButton_yolo2isat_yolo_image_root.setObjectName("pushButton_yolo2isat_yolo_image_root")
        self.gridLayout_3.addWidget(self.pushButton_yolo2isat_yolo_image_root, 0, 2, 1, 1)
        self.lineEdit_yolo2isat_yolo_cate_path = QtWidgets.QLineEdit(self.widget_11)
        self.lineEdit_yolo2isat_yolo_cate_path.setText("")
        self.lineEdit_yolo2isat_yolo_cate_path.setReadOnly(True)
        self.lineEdit_yolo2isat_yolo_cate_path.setObjectName("lineEdit_yolo2isat_yolo_cate_path")
        self.gridLayout_3.addWidget(self.lineEdit_yolo2isat_yolo_cate_path, 2, 0, 1, 1)
        self.pushButton_yolo2isat_yolo_cate_path = QtWidgets.QPushButton(self.widget_11)
        self.pushButton_yolo2isat_yolo_cate_path.setObjectName("pushButton_yolo2isat_yolo_cate_path")
        self.gridLayout_3.addWidget(self.pushButton_yolo2isat_yolo_cate_path, 2, 2, 1, 1)
        self.label = QtWidgets.QLabel(self.widget_11)
        self.label.setToolTip("")
        self.label.setStatusTip("")
        self.label.setWhatsThis("")
        self.label.setStyleSheet("color: rgb(255, 0, 0);")
        self.label.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label.setObjectName("label")
        self.gridLayout_3.addWidget(self.label, 0, 1, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.widget_11)
        self.label_7.setStyleSheet("color: rgb(255, 0, 0);")
        self.label_7.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_7.setObjectName("label_7")
        self.gridLayout_3.addWidget(self.label_7, 1, 1, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.widget_11)
        self.label_8.setText("")
        self.label_8.setObjectName("label_8")
        self.gridLayout_3.addWidget(self.label_8, 2, 1, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.widget_11)
        self.label_9.setStyleSheet("color: rgb(255, 0, 0);")
        self.label_9.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_9.setObjectName("label_9")
        self.gridLayout_3.addWidget(self.label_9, 3, 1, 1, 1)
        self.verticalLayout_10.addWidget(self.widget_11)
        self.verticalLayout_9.addWidget(self.widget_10)
        self.toolBox_yolo.addItem(self.toolbox_item_yolo2isat, "")
        self.toolbox_item_isat2yolo = QtWidgets.QWidget()
        self.toolbox_item_isat2yolo.setGeometry(QtCore.QRect(0, 0, 960, 153))
        self.toolbox_item_isat2yolo.setObjectName("toolbox_item_isat2yolo")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.toolbox_item_isat2yolo)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.widget_13 = QtWidgets.QWidget(self.toolbox_item_isat2yolo)
        self.widget_13.setObjectName("widget_13")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.widget_13)
        self.verticalLayout_12.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.widget_14 = QtWidgets.QWidget(self.widget_13)
        self.widget_14.setObjectName("widget_14")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.widget_14)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.pushButton_isat2yolo_yolo_txt_root = QtWidgets.QPushButton(self.widget_14)
        self.pushButton_isat2yolo_yolo_txt_root.setObjectName("pushButton_isat2yolo_yolo_txt_root")
        self.gridLayout_4.addWidget(self.pushButton_isat2yolo_yolo_txt_root, 1, 1, 1, 1)
        self.lineEdit_isat2yolo_yolo_txt_root = QtWidgets.QLineEdit(self.widget_14)
        self.lineEdit_isat2yolo_yolo_txt_root.setReadOnly(True)
        self.lineEdit_isat2yolo_yolo_txt_root.setObjectName("lineEdit_isat2yolo_yolo_txt_root")
        self.gridLayout_4.addWidget(self.lineEdit_isat2yolo_yolo_txt_root, 1, 0, 1, 1)
        self.lineEdit_isat2yolo_isat_json_root = QtWidgets.QLineEdit(self.widget_14)
        self.lineEdit_isat2yolo_isat_json_root.setReadOnly(True)
        self.lineEdit_isat2yolo_isat_json_root.setObjectName("lineEdit_isat2yolo_isat_json_root")
        self.gridLayout_4.addWidget(self.lineEdit_isat2yolo_isat_json_root, 0, 0, 1, 1)
        self.pushButton_isat2yolo_isat_json_root = QtWidgets.QPushButton(self.widget_14)
        self.pushButton_isat2yolo_isat_json_root.setObjectName("pushButton_isat2yolo_isat_json_root")
        self.gridLayout_4.addWidget(self.pushButton_isat2yolo_isat_json_root, 0, 1, 1, 1)
        self.verticalLayout_12.addWidget(self.widget_14)
        self.verticalLayout_11.addWidget(self.widget_13)
        self.toolBox_yolo.addItem(self.toolbox_item_isat2yolo, "")
        self.verticalLayout_3.addWidget(self.toolBox_yolo)
        self.label_3 = QtWidgets.QLabel(self.widget_9)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.label_3.setFont(font)
        self.label_3.setStyleSheet("color: rgb(255, 0, 0);")
        self.label_3.setObjectName("label_3")
        self.verticalLayout_3.addWidget(self.label_3)
        self.verticalLayout_8.addWidget(self.widget_9)
        self.tabWidget.addTab(self.tab_YOLO, "")
        self.tab_LABELME = QtWidgets.QWidget()
        self.tab_LABELME.setObjectName("tab_LABELME")
        self.verticalLayout_20 = QtWidgets.QVBoxLayout(self.tab_LABELME)
        self.verticalLayout_20.setObjectName("verticalLayout_20")
        self.widget_16 = QtWidgets.QWidget(self.tab_LABELME)
        self.widget_16.setObjectName("widget_16")
        self.verticalLayout_15 = QtWidgets.QVBoxLayout(self.widget_16)
        self.verticalLayout_15.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_15.setObjectName("verticalLayout_15")
        self.toolBox_labelme = QtWidgets.QToolBox(self.widget_16)
        self.toolBox_labelme.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.toolBox_labelme.setFrameShadow(QtWidgets.QFrame.Plain)
        self.toolBox_labelme.setObjectName("toolBox_labelme")
        self.toolbox_item_labelme2isat = QtWidgets.QWidget()
        self.toolbox_item_labelme2isat.setGeometry(QtCore.QRect(0, 0, 960, 153))
        self.toolbox_item_labelme2isat.setObjectName("toolbox_item_labelme2isat")
        self.verticalLayout_16 = QtWidgets.QVBoxLayout(self.toolbox_item_labelme2isat)
        self.verticalLayout_16.setContentsMargins(9, 9, 9, 9)
        self.verticalLayout_16.setObjectName("verticalLayout_16")
        self.widget_17 = QtWidgets.QWidget(self.toolbox_item_labelme2isat)
        self.widget_17.setObjectName("widget_17")
        self.verticalLayout_17 = QtWidgets.QVBoxLayout(self.widget_17)
        self.verticalLayout_17.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_17.setObjectName("verticalLayout_17")
        self.widget_18 = QtWidgets.QWidget(self.widget_17)
        self.widget_18.setObjectName("widget_18")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.widget_18)
        self.gridLayout_6.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.pushButton_labelme2isat_labelme_json_root = QtWidgets.QPushButton(self.widget_18)
        self.pushButton_labelme2isat_labelme_json_root.setObjectName("pushButton_labelme2isat_labelme_json_root")
        self.gridLayout_6.addWidget(self.pushButton_labelme2isat_labelme_json_root, 0, 1, 1, 1)
        self.lineEdit_labelme2isat_isat_json_root = QtWidgets.QLineEdit(self.widget_18)
        self.lineEdit_labelme2isat_isat_json_root.setReadOnly(True)
        self.lineEdit_labelme2isat_isat_json_root.setObjectName("lineEdit_labelme2isat_isat_json_root")
        self.gridLayout_6.addWidget(self.lineEdit_labelme2isat_isat_json_root, 1, 0, 1, 1)
        self.lineEdit_labelme2isat_labelme_json_root = QtWidgets.QLineEdit(self.widget_18)
        self.lineEdit_labelme2isat_labelme_json_root.setReadOnly(True)
        self.lineEdit_labelme2isat_labelme_json_root.setObjectName("lineEdit_labelme2isat_labelme_json_root")
        self.gridLayout_6.addWidget(self.lineEdit_labelme2isat_labelme_json_root, 0, 0, 1, 1)
        self.pushButton_labelme2isat_isat_json_root = QtWidgets.QPushButton(self.widget_18)
        self.pushButton_labelme2isat_isat_json_root.setObjectName("pushButton_labelme2isat_isat_json_root")
        self.gridLayout_6.addWidget(self.pushButton_labelme2isat_isat_json_root, 1, 1, 1, 1)
        self.verticalLayout_17.addWidget(self.widget_18)
        self.verticalLayout_16.addWidget(self.widget_17)
        self.toolBox_labelme.addItem(self.toolbox_item_labelme2isat, "")
        self.toolbox_item_isat2labelme = QtWidgets.QWidget()
        self.toolbox_item_isat2labelme.setGeometry(QtCore.QRect(0, 0, 138, 78))
        self.toolbox_item_isat2labelme.setObjectName("toolbox_item_isat2labelme")
        self.verticalLayout_18 = QtWidgets.QVBoxLayout(self.toolbox_item_isat2labelme)
        self.verticalLayout_18.setObjectName("verticalLayout_18")
        self.widget_20 = QtWidgets.QWidget(self.toolbox_item_isat2labelme)
        self.widget_20.setObjectName("widget_20")
        self.verticalLayout_19 = QtWidgets.QVBoxLayout(self.widget_20)
        self.verticalLayout_19.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_19.setObjectName("verticalLayout_19")
        self.widget_21 = QtWidgets.QWidget(self.widget_20)
        self.widget_21.setObjectName("widget_21")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.widget_21)
        self.gridLayout_7.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.lineEdit_isat2labelme_isat_json_root = QtWidgets.QLineEdit(self.widget_21)
        self.lineEdit_isat2labelme_isat_json_root.setReadOnly(True)
        self.lineEdit_isat2labelme_isat_json_root.setObjectName("lineEdit_isat2labelme_isat_json_root")
        self.gridLayout_7.addWidget(self.lineEdit_isat2labelme_isat_json_root, 0, 0, 1, 1)
        self.pushButton_isat2labelme_isat_json_root = QtWidgets.QPushButton(self.widget_21)
        self.pushButton_isat2labelme_isat_json_root.setObjectName("pushButton_isat2labelme_isat_json_root")
        self.gridLayout_7.addWidget(self.pushButton_isat2labelme_isat_json_root, 0, 1, 1, 1)
        self.lineEdit_isat2labelme_labelme_json_root = QtWidgets.QLineEdit(self.widget_21)
        self.lineEdit_isat2labelme_labelme_json_root.setReadOnly(True)
        self.lineEdit_isat2labelme_labelme_json_root.setObjectName("lineEdit_isat2labelme_labelme_json_root")
        self.gridLayout_7.addWidget(self.lineEdit_isat2labelme_labelme_json_root, 1, 0, 1, 1)
        self.pushButton_isat2labelme_labelme_json_root = QtWidgets.QPushButton(self.widget_21)
        self.pushButton_isat2labelme_labelme_json_root.setObjectName("pushButton_isat2labelme_labelme_json_root")
        self.gridLayout_7.addWidget(self.pushButton_isat2labelme_labelme_json_root, 1, 1, 1, 1)
        self.verticalLayout_19.addWidget(self.widget_21)
        self.verticalLayout_18.addWidget(self.widget_20)
        self.toolBox_labelme.addItem(self.toolbox_item_isat2labelme, "")
        self.verticalLayout_15.addWidget(self.toolBox_labelme)
        self.verticalLayout_20.addWidget(self.widget_16)
        self.label_4 = QtWidgets.QLabel(self.tab_LABELME)
        self.label_4.setStyleSheet("color: rgb(255, 0, 0);")
        self.label_4.setObjectName("label_4")
        self.verticalLayout_20.addWidget(self.label_4)
        self.tabWidget.addTab(self.tab_LABELME, "")
        self.tab_VOC = QtWidgets.QWidget()
        self.tab_VOC.setObjectName("tab_VOC")
        self.verticalLayout_14 = QtWidgets.QVBoxLayout(self.tab_VOC)
        self.verticalLayout_14.setObjectName("verticalLayout_14")
        self.widget_12 = QtWidgets.QWidget(self.tab_VOC)
        self.widget_12.setObjectName("widget_12")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.widget_12)
        self.gridLayout_5.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.pushButton_isat2voc_voc_png_root = QtWidgets.QPushButton(self.widget_12)
        self.pushButton_isat2voc_voc_png_root.setObjectName("pushButton_isat2voc_voc_png_root")
        self.gridLayout_5.addWidget(self.pushButton_isat2voc_voc_png_root, 1, 1, 1, 1)
        self.lineEdit_isat2voc_voc_png_root = QtWidgets.QLineEdit(self.widget_12)
        self.lineEdit_isat2voc_voc_png_root.setReadOnly(True)
        self.lineEdit_isat2voc_voc_png_root.setObjectName("lineEdit_isat2voc_voc_png_root")
        self.gridLayout_5.addWidget(self.lineEdit_isat2voc_voc_png_root, 1, 0, 1, 1)
        self.lineEdit_isat2voc_isat_json_root = QtWidgets.QLineEdit(self.widget_12)
        self.lineEdit_isat2voc_isat_json_root.setReadOnly(True)
        self.lineEdit_isat2voc_isat_json_root.setObjectName("lineEdit_isat2voc_isat_json_root")
        self.gridLayout_5.addWidget(self.lineEdit_isat2voc_isat_json_root, 0, 0, 1, 1)
        self.pushButton_isat2voc_isat_json_root = QtWidgets.QPushButton(self.widget_12)
        self.pushButton_isat2voc_isat_json_root.setObjectName("pushButton_isat2voc_isat_json_root")
        self.gridLayout_5.addWidget(self.pushButton_isat2voc_isat_json_root, 0, 1, 1, 1)
        self.verticalLayout_14.addWidget(self.widget_12)
        self.widget_15 = QtWidgets.QWidget(self.tab_VOC)
        self.widget_15.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.widget_15.setObjectName("widget_15")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.widget_15)
        self.horizontalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem2 = QtWidgets.QSpacerItem(708, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem2)
        self.checkBox_is_instance = QtWidgets.QCheckBox(self.widget_15)
        self.checkBox_is_instance.setObjectName("checkBox_is_instance")
        self.horizontalLayout_4.addWidget(self.checkBox_is_instance)
        self.verticalLayout_14.addWidget(self.widget_15)
        self.widget_22 = QtWidgets.QWidget(self.tab_VOC)
        self.widget_22.setMinimumSize(QtCore.QSize(0, 0))
        self.widget_22.setMaximumSize(QtCore.QSize(16777215, 20))
        self.widget_22.setObjectName("widget_22")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.widget_22)
        self.horizontalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_5.setSpacing(0)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_5 = QtWidgets.QLabel(self.widget_22)
        self.label_5.setMaximumSize(QtCore.QSize(16777215, 20))
        self.label_5.setStyleSheet("color: rgb(255, 0, 0);")
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_5.addWidget(self.label_5)
        self.label_10 = QtWidgets.QLabel(self.widget_22)
        self.label_10.setStyleSheet("color: rgb(255, 0, 0);")
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_5.addWidget(self.label_10)
        self.verticalLayout_14.addWidget(self.widget_22)
        self.tabWidget.addTab(self.tab_VOC, "")
        self.tab_VOC_DETECTION = QtWidgets.QWidget()
        self.tab_VOC_DETECTION.setObjectName("tab_VOC_DETECTION")
        self.verticalLayout_21 = QtWidgets.QVBoxLayout(self.tab_VOC_DETECTION)
        self.verticalLayout_21.setObjectName("verticalLayout_21")
        self.widget_19 = QtWidgets.QWidget(self.tab_VOC_DETECTION)
        self.widget_19.setObjectName("widget_19")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.widget_19)
        self.gridLayout_8.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.lineEdit_isat2vocod_isat_json_root = QtWidgets.QLineEdit(self.widget_19)
        self.lineEdit_isat2vocod_isat_json_root.setReadOnly(True)
        self.lineEdit_isat2vocod_isat_json_root.setObjectName("lineEdit_isat2vocod_isat_json_root")
        self.gridLayout_8.addWidget(self.lineEdit_isat2vocod_isat_json_root, 0, 0, 1, 1)
        self.pushButton_isat2vocod_isat_json_root = QtWidgets.QPushButton(self.widget_19)
        self.pushButton_isat2vocod_isat_json_root.setObjectName("pushButton_isat2vocod_isat_json_root")
        self.gridLayout_8.addWidget(self.pushButton_isat2vocod_isat_json_root, 0, 1, 1, 1)
        self.lineEdit_isat2vocod_voc_xml_root = QtWidgets.QLineEdit(self.widget_19)
        self.lineEdit_isat2vocod_voc_xml_root.setReadOnly(True)
        self.lineEdit_isat2vocod_voc_xml_root.setObjectName("lineEdit_isat2vocod_voc_xml_root")
        self.gridLayout_8.addWidget(self.lineEdit_isat2vocod_voc_xml_root, 1, 0, 1, 1)
        self.pushButton_isat2vocod_voc_xml_root = QtWidgets.QPushButton(self.widget_19)
        self.pushButton_isat2vocod_voc_xml_root.setObjectName("pushButton_isat2vocod_voc_xml_root")
        self.gridLayout_8.addWidget(self.pushButton_isat2vocod_voc_xml_root, 1, 1, 1, 1)
        self.verticalLayout_21.addWidget(self.widget_19)
        self.label_6 = QtWidgets.QLabel(self.tab_VOC_DETECTION)
        self.label_6.setMaximumSize(QtCore.QSize(16777215, 20))
        self.label_6.setStyleSheet("color: rgb(255, 0, 0);")
        self.label_6.setObjectName("label_6")
        self.verticalLayout_21.addWidget(self.label_6)
        self.tabWidget.addTab(self.tab_VOC_DETECTION, "")
        self.verticalLayout_13.addWidget(self.tabWidget)
        self.textBrowser = QtWidgets.QTextBrowser(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.textBrowser.sizePolicy().hasHeightForWidth())
        self.textBrowser.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("宋体")
        self.textBrowser.setFont(font)
        self.textBrowser.setObjectName("textBrowser")
        self.verticalLayout_13.addWidget(self.textBrowser)
        self.progressBar = QtWidgets.QProgressBar(Dialog)
        self.progressBar.setProperty("value", 0)
        self.progressBar.setObjectName("progressBar")
        self.verticalLayout_13.addWidget(self.progressBar)
        self.widget_2 = QtWidgets.QWidget(Dialog)
        self.widget_2.setObjectName("widget_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget_2)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem3 = QtWidgets.QSpacerItem(308, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem3)
        self.pushButton_cancel = QtWidgets.QPushButton(self.widget_2)
        self.pushButton_cancel.setObjectName("pushButton_cancel")
        self.horizontalLayout.addWidget(self.pushButton_cancel)
        self.pushButton_convert = QtWidgets.QPushButton(self.widget_2)
        self.pushButton_convert.setObjectName("pushButton_convert")
        self.horizontalLayout.addWidget(self.pushButton_convert)
        self.verticalLayout_13.addWidget(self.widget_2)

        self.retranslateUi(Dialog)
        self.tabWidget.setCurrentIndex(3)
        self.toolBox_coco.setCurrentIndex(1)
        self.toolBox_yolo.setCurrentIndex(1)
        self.toolBox_labelme.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Converter"))
        self.pushButton_coco2isat_coco_json_path.setText(_translate("Dialog", "json path"))
        self.lineEdit_coco2isat_isat_json_root.setPlaceholderText(_translate("Dialog", "ISAT jsons save root"))
        self.lineEdit_coco2isat_coco_json_path.setPlaceholderText(_translate("Dialog", "COCO json path"))
        self.pushButton_coco2isat_isat_json_root.setText(_translate("Dialog", "save root"))
        self.checkBox_coco2isat_keep_crowd.setText(_translate("Dialog", "keep crowd"))
        self.toolBox_coco.setItemText(self.toolBox_coco.indexOf(self.toolbox_item_coco2isat), _translate("Dialog", "COCO to ISAT"))
        self.lineEdit_isat2coco_isat_json_root.setPlaceholderText(_translate("Dialog", "ISAT jsons root"))
        self.pushButton_isat2coco_isat_json_root.setText(_translate("Dialog", "json root"))
        self.lineEdit_isat2coco_coco_json_path.setPlaceholderText(_translate("Dialog", "COCO json save path"))
        self.pushButton_isat2coco_coco_json_path.setText(_translate("Dialog", "save path"))
        self.checkBox_isat2coco_keep_crowd.setText(_translate("Dialog", "keep crowd"))
        self.toolBox_coco.setItemText(self.toolBox_coco.indexOf(self.toolbox_item_isat2coco), _translate("Dialog", "ISAT to COCO"))
        self.label_2.setText(_translate("Dialog", "COCO save annotations to a single JSON file."))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_COCO), _translate("Dialog", "COCO"))
        self.lineEdit_yolo2isat_isat_json_root.setPlaceholderText(_translate("Dialog", "ISAT jsons save root"))
        self.pushButton_yolo2isat_isat_json_root.setText(_translate("Dialog", "save root"))
        self.lineEdit_yolo2isat_yolo_txt_root.setPlaceholderText(_translate("Dialog", "YOLO txts root"))
        self.pushButton_yolo2isat_yolo_txt_root.setText(_translate("Dialog", "txt root"))
        self.lineEdit_yolo2isat_yolo_image_root.setPlaceholderText(_translate("Dialog", "YOLO images root"))
        self.pushButton_yolo2isat_yolo_image_root.setText(_translate("Dialog", "image root"))
        self.lineEdit_yolo2isat_yolo_cate_path.setPlaceholderText(_translate("Dialog", "YOLO category file path.(Txt file with one line for one category.)  "))
        self.pushButton_yolo2isat_yolo_cate_path.setText(_translate("Dialog", "category file"))
        self.label.setText(_translate("Dialog", "*"))
        self.label_7.setText(_translate("Dialog", "*"))
        self.label_9.setText(_translate("Dialog", "*"))
        self.toolBox_yolo.setItemText(self.toolBox_yolo.indexOf(self.toolbox_item_yolo2isat), _translate("Dialog", "YOLO to ISAT"))
        self.pushButton_isat2yolo_yolo_txt_root.setText(_translate("Dialog", "save root"))
        self.lineEdit_isat2yolo_yolo_txt_root.setPlaceholderText(_translate("Dialog", "YOLO txts save root"))
        self.lineEdit_isat2yolo_isat_json_root.setPlaceholderText(_translate("Dialog", "ISAT jsons root"))
        self.pushButton_isat2yolo_isat_json_root.setText(_translate("Dialog", "json root"))
        self.toolBox_yolo.setItemText(self.toolBox_yolo.indexOf(self.toolbox_item_isat2yolo), _translate("Dialog", "ISAT to YOLO"))
        self.label_3.setText(_translate("Dialog", "YOLO save annotations to multiple TXT files."))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_YOLO), _translate("Dialog", "YOLO"))
        self.pushButton_labelme2isat_labelme_json_root.setText(_translate("Dialog", "json root"))
        self.lineEdit_labelme2isat_isat_json_root.setPlaceholderText(_translate("Dialog", "ISAT jsons save root"))
        self.lineEdit_labelme2isat_labelme_json_root.setPlaceholderText(_translate("Dialog", "LABELME jsons root"))
        self.pushButton_labelme2isat_isat_json_root.setText(_translate("Dialog", "save root"))
        self.toolBox_labelme.setItemText(self.toolBox_labelme.indexOf(self.toolbox_item_labelme2isat), _translate("Dialog", "LABELME to ISAT"))
        self.lineEdit_isat2labelme_isat_json_root.setPlaceholderText(_translate("Dialog", "ISAT jsons root"))
        self.pushButton_isat2labelme_isat_json_root.setText(_translate("Dialog", "json root"))
        self.lineEdit_isat2labelme_labelme_json_root.setPlaceholderText(_translate("Dialog", "LABELME jsons save root"))
        self.pushButton_isat2labelme_labelme_json_root.setText(_translate("Dialog", "save root"))
        self.toolBox_labelme.setItemText(self.toolBox_labelme.indexOf(self.toolbox_item_isat2labelme), _translate("Dialog", "ISAT to LABELME"))
        self.label_4.setText(_translate("Dialog", "LABELME save annotations to multiple JSON files. (When convert LABELME to ISAT for instance segmentation, labelme group id must>0.)"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_LABELME), _translate("Dialog", "LABELME"))
        self.pushButton_isat2voc_voc_png_root.setText(_translate("Dialog", "save root"))
        self.lineEdit_isat2voc_voc_png_root.setPlaceholderText(_translate("Dialog", "VOC pngs save root"))
        self.lineEdit_isat2voc_isat_json_root.setPlaceholderText(_translate("Dialog", "ISAT jsons root"))
        self.pushButton_isat2voc_isat_json_root.setText(_translate("Dialog", "json root"))
        self.checkBox_is_instance.setText(_translate("Dialog", "is instance"))
        self.label_5.setText(_translate("Dialog", "VOC save annotations to multiple PNG files."))
        self.label_10.setText(_translate("Dialog", "**The num of classification and the group must in [0, 255]**"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_VOC), _translate("Dialog", "VOC"))
        self.lineEdit_isat2vocod_isat_json_root.setPlaceholderText(_translate("Dialog", "ISAT jsons root"))
        self.pushButton_isat2vocod_isat_json_root.setText(_translate("Dialog", "json root"))
        self.lineEdit_isat2vocod_voc_xml_root.setPlaceholderText(_translate("Dialog", "VOC xmls save root"))
        self.pushButton_isat2vocod_voc_xml_root.setText(_translate("Dialog", "save root"))
        self.label_6.setText(_translate("Dialog", "VOC save annotations to multiple XML files for object detection."))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_VOC_DETECTION), _translate("Dialog", "VOC for object detection"))
        self.pushButton_cancel.setText(_translate("Dialog", "Cancel"))
        self.pushButton_convert.setText(_translate("Dialog", "Convert"))



# -*- coding: utf-8 -*-
# @Author  : LG

from ISAT.scripts.isat import ISAT
import cv2
import tqdm
import numpy as np
import os


class YOLO(ISAT):
    """
    YOLO use txt to save annotations. Every line container a annotation.
    format: class_index, x1, y1, x2, y2, x3, y3, ....

    save_to_YOLO:
        针对多个一个目标包含多个多边形的情况，参考了yolo8 coco2yolo的转换代码，在多边形之间拉一条直线，将多个多边形组合为单个多边形进行保存。
        源代码地址: https://github.com/ultralytics/JSON2YOLO/blob/c38a43f342428849c75c103c6d060012a83b5392/general_json2yolo.py
    """
    def __init__(self):
        self.keep_crowd = True

    def read_from_YOLO(self, img_root, txt_root, class_dict=None):
        self.annos.clear()
        self.cates = ()

        img_files = os.listdir(img_root)

        pbar = tqdm.tqdm(img_files)
        for img_name in pbar:
            name_without_suffix = self.remove_file_suffix(img_name)
            txt_path = os.path.join(txt_root, name_without_suffix+'.txt')
            image_path = os.path.join(img_root, img_name)
            if not os.path.exists(txt_path):
                continue

            anno = self._load_one_yolo_txt(image_path, txt_path, class_dict)
            self.annos[name_without_suffix] = anno
            pbar.set_description('Load yolo txt {}'.format(name_without_suffix+'.txt'))

        # cates
        if class_dict is not None:
            self.cates = tuple(class_dict.values())
        else:
            class_set = set()
            for _, anno in self.annos.items():
                for obj in anno.objs:
                    class_set.add(obj.category)
            class_set = list(class_set)
            class_set.sort()
            self.cates = tuple(class_set)
        return True

    def save_to_YOLO(self, txt_root):
        os.makedirs(txt_root, exist_ok=True)
        cates_index_dict = {cat:index for index, cat in enumerate(self.cates)}

        with open(os.path.join(txt_root, 'classification.txt'), 'w') as f:
            for cat in self.cates:
                f.write('{}\n'.format(cat))

        pbar = tqdm.tqdm(self.annos.items())
        for name_without_suffix, anno in pbar:
            txt_path = os.path.join(txt_root, name_without_suffix+'.txt')
            pbar.set_description('Integrate {}'.format(name_without_suffix))
            try:
                self._save_one_yolo_txt(anno, txt_path, cates_index_dict)
                pbar.set_description('Save yolo to {}'.format(name_without_suffix+'.txt'))

            except Exception as e:
                raise '{} {}'.format(name_without_suffix, e)
        return True

    @staticmethod
    def merge_multi_segment(segments):
        """
        https://github.com/ultralytics/JSON2YOLO/blob/c38a43f342428849c75c103c6d060012a83b5392/general_json2yolo.py#L324

        Merge multi segments to one list.
        Find the coordinates with min distance between each segment,
        then connect these coordinates with one thin line to merge all
        segments into one.

        Args:
            segments(List(List)): original segmentations in coco's json file.
                like [segmentation1, segmentation2,...],
                each segmentation is a list of coordinates.
        """

        def min_index(arr1, arr2):
            """
            https://github.com/ultralytics/JSON2YOLO/blob/c38a43f342428849c75c103c6d060012a83b5392/general_json2yolo.py#L324
            Find a pair of indexes with the shortest distance.
            Args:
                arr1: (N, 2).
                arr2: (M, 2).
            Return:
                a pair of indexes(tuple).
            """
            dis = ((arr1[:, None, :] - arr2[None, :, :]) ** 2).sum(-1)
            return np.unravel_index(np.argmin(dis, axis=None), dis.shape)

        s = []
        segments = [np.array(i).reshape(-1, 2) for i in segments]
        idx_list = [[] for _ in range(len(segments))]

        # record the indexes with min distance between each segment
        for i in range(1, len(segments)):
            idx1, idx2 = min_index(segments[i - 1], segments[i])
            idx_list[i - 1].append(idx1)
            idx_list[i].append(idx2)

        # use two round to connect all the segments
        for k in range(2):
            # forward connection
            if k == 0:
                for i, idx in enumerate(idx_list):
                    # middle segments have two indexes
                    # reverse the index of middle segments
                    if len(idx) == 2 and idx[0] > idx[1]:
                        idx = idx[::-1]
                        segments[i] = segments[i][::-1, :]

                    segments[i] = np.roll(segments[i], -idx[0], axis=0)
                    segments[i] = np.concatenate([segments[i], segments[i][:1]])
                    # deal with the first segment and the last one
                    if i in [0, len(idx_list) - 1]:
                        s.append(segments[i])
                    else:
                        idx = [0, idx[1] - idx[0]]
                        s.append(segments[i][idx[0]:idx[1] + 1])

            else:
                for i in range(len(idx_list) - 1, -1, -1):
                    if i not in [0, len(idx_list) - 1]:
                        idx = idx_list[i]
                        nidx = abs(idx[1] - idx[0])
                        s.append(segments[i][nidx:])
        return s

    @staticmethod
    def yolo2isat_segmentation(yolo_seg, img_width, img_height):
        """Convert YOLO segmentation format to ISAT segmentation format"""
        return [[round(x * img_width), round(y * img_height)] for x, y in zip(yolo_seg[::2], yolo_seg[1::2])]

    @staticmethod
    def get_isat_bbox(segmentation):
        """Calculate the bbox from the ISAT segmentation"""
        xs = [point[0] for point in segmentation]  # x-coordinates
        ys = [point[1] for point in segmentation]  # y-coordinates
        return [int(min(xs)), int(min(ys)), int(max(xs)), int(max(ys))]

    @staticmethod
    def bbox_within(bbox_1, bbox_2):  # 这个函数查看两个物体的边框，如果是包含关系的话分到一个组
        """Check if two objects belong to the same group"""
        return all(bbox_1[idx] >= bbox_2[idx] for idx in [0, 1]) and all(bbox_1[idx] <= bbox_2[idx] for idx in [2, 3])

    def _load_one_yolo_txt(self, image_path, txt_path, class_dict=None):

        anno = self.ANNO()
        anno.info = self.ANNO.INFO()

        image = cv2.imread(image_path)  # load the image in BRG scale

        image_width, image_height = image.shape[1], image.shape[0]  # get the image dimensions

        if image.ndim == 2:
            image_depth = 1
        elif image.ndim == 3:
            image_depth = image.shape[2]
        else:
            image_depth = None

        img_root, img_name = os.path.split(image_path)
        anno.info.description = ''
        anno.info.folder = img_root
        anno.info.name = img_name
        anno.info.width = image_width
        anno.info.height = image_height
        anno.info.depth = image_depth

        objects = []
        group, layer = 1, 1.0  # initialize layer as a floating point number

        with open(txt_path, 'r', encoding='utf-8') as f:
            lines = f.readlines()
            for line in lines:
                obj = self.ANNO.OBJ()

                parts = line.split()  # split each line
                class_index = int(parts[0])  # get the class index
                yolo_segmentation = list(map(float, parts[1:]))  # get the yolo_segmentation
                isat_segmentation = self.yolo2isat_segmentation(yolo_segmentation, image_width,
                                                                image_height)  # convert yolo_segmentation to isat_segmentation
                bbox = self.get_isat_bbox(isat_segmentation)  # calculate the bbox from segmentation
                area = (bbox[2] - bbox[0]) * (bbox[3] - bbox[
                    1])  # roughly calculate the bbox area as segmentation area, it will be replaced anyway

                obj.category = class_index if class_dict is None else class_dict.get(class_index, 'UNKNOW')
                obj.group = group
                obj.segmentation = isat_segmentation
                obj.area = area
                obj.layer = layer
                obj.bbox = bbox
                obj.iscrowd = False

                group += 1
                objects.append(obj)

        anno.objs = tuple(objects)
        return anno

    def _save_one_yolo_txt(self, anno, save_path, cates_index_dict):
        with open(save_path, 'w') as f:
            objects = anno.objs

            objects_groups = [obj.group for obj in objects]
            objects_groups.sort()
            objects_groups = set(objects_groups)
            # 同group
            for group_index, group in enumerate(objects_groups):
                objs_with_group = [obj for obj in objects if obj.group == group]
                cats = [obj.category for obj in objs_with_group]
                cats = set(cats)

                # 同category
                for cat in cats:
                    objs_with_cat = [obj for obj in objs_with_group if obj.category == cat]
                    crowds = [obj.iscrowd for obj in objs_with_group]
                    crowds = set(crowds)
                    # 同iscrowd
                    class_index = cates_index_dict.get(cat)
                    segmentations = []

                    for obj in objs_with_cat:
                        if not self.keep_crowd and obj.iscrowd:
                            continue
                        segmentation = [[x / anno.info.width, y / anno.info.height] for x, y in obj.segmentation]
                        segmentation = [c for xy in segmentation for c in xy]
                        segmentations.append(segmentation)

                    if len(segmentations) > 1:
                        segmentations = self.merge_multi_segment(segmentations)
                        segmentations = (np.concatenate(segmentations, axis=0)).reshape(-1).tolist()
                    else:
                        segmentations = segmentations[0]

                    s = '{}' + ' {}' * len(segmentations) + '\n'
                    f.write(s.format(class_index, *segmentations))
        return True


# Copyright (c) Meta Platforms, Inc. and affiliates.
# All rights reserved.

# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

import torch
import torch.nn as nn

from typing import Type


class MLPBlock(nn.Module):
    def __init__(
        self,
        embedding_dim: int,
        mlp_dim: int,
        act: Type[nn.Module] = nn.GELU,
    ) -> None:
        super().__init__()
        self.lin1 = nn.Linear(embedding_dim, mlp_dim)
        self.lin2 = nn.Linear(mlp_dim, embedding_dim)
        self.act = act()

    def forward(self, x: torch.Tensor) -> torch.Tensor:
        return self.lin2(self.act(self.lin1(x)))


# From https://github.com/facebookresearch/detectron2/blob/main/detectron2/layers/batch_norm.py # noqa
# Itself from https://github.com/facebookresearch/ConvNeXt/blob/d1fa8f6fef0a165b27399986cc2bdacc92777e40/models/convnext.py#L119  # noqa
class LayerNorm2d(nn.Module):
    def __init__(self, num_channels: int, eps: float = 1e-6) -> None:
        super().__init__()
        self.weight = nn.Parameter(torch.ones(num_channels))
        self.bias = nn.Parameter(torch.zeros(num_channels))
        self.eps = eps

    def forward(self, x: torch.Tensor) -> torch.Tensor:
        u = x.mean(-3, keepdim=True)
        s = (x - u).pow(2).mean(-3, keepdim=True)
        x = (x - u) / torch.sqrt(s + self.eps)
        x = self.weight[:, None, None] * x + self.bias[:, None, None]
        return x



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/lg/PycharmProjects/ISAT_with_segment_anything/ISAT/ui/category_choice.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(350, 399)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        Dialog.setFont(font)
        Dialog.setWindowTitle("")
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.listWidget = QtWidgets.QListWidget(Dialog)
        self.listWidget.setObjectName("listWidget")
        self.verticalLayout.addWidget(self.listWidget)
        self.widget_3 = QtWidgets.QWidget(Dialog)
        self.widget_3.setObjectName("widget_3")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.widget_3)
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_2 = QtWidgets.QLabel(self.widget_3)
        self.label_2.setMinimumSize(QtCore.QSize(60, 0))
        self.label_2.setMaximumSize(QtCore.QSize(60, 16777215))
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_3.addWidget(self.label_2)
        self.lineEdit_category = QtWidgets.QLineEdit(self.widget_3)
        self.lineEdit_category.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lineEdit_category.setReadOnly(True)
        self.lineEdit_category.setObjectName("lineEdit_category")
        self.horizontalLayout_3.addWidget(self.lineEdit_category)
        self.label = QtWidgets.QLabel(self.widget_3)
        self.label.setMinimumSize(QtCore.QSize(50, 0))
        self.label.setMaximumSize(QtCore.QSize(50, 16777215))
        self.label.setObjectName("label")
        self.horizontalLayout_3.addWidget(self.label)
        self.spinBox_group = QtWidgets.QSpinBox(self.widget_3)
        self.spinBox_group.setMaximum(9999)
        self.spinBox_group.setObjectName("spinBox_group")
        self.horizontalLayout_3.addWidget(self.spinBox_group)
        self.verticalLayout.addWidget(self.widget_3)
        self.widget_5 = QtWidgets.QWidget(Dialog)
        self.widget_5.setObjectName("widget_5")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.widget_5)
        self.horizontalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_3 = QtWidgets.QLabel(self.widget_5)
        self.label_3.setMinimumSize(QtCore.QSize(60, 0))
        self.label_3.setMaximumSize(QtCore.QSize(60, 16777215))
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_5.addWidget(self.label_3)
        self.lineEdit_note = QtWidgets.QLineEdit(self.widget_5)
        self.lineEdit_note.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lineEdit_note.setObjectName("lineEdit_note")
        self.horizontalLayout_5.addWidget(self.lineEdit_note)
        self.label_4 = QtWidgets.QLabel(self.widget_5)
        self.label_4.setMinimumSize(QtCore.QSize(50, 0))
        self.label_4.setMaximumSize(QtCore.QSize(50, 16777215))
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_5.addWidget(self.label_4)
        self.label_layer = QtWidgets.QLabel(self.widget_5)
        self.label_layer.setMinimumSize(QtCore.QSize(60, 0))
        self.label_layer.setMaximumSize(QtCore.QSize(60, 16777215))
        self.label_layer.setText("")
        self.label_layer.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_layer.setObjectName("label_layer")
        self.horizontalLayout_5.addWidget(self.label_layer)
        self.verticalLayout.addWidget(self.widget_5)
        self.widget = QtWidgets.QWidget(Dialog)
        self.widget.setObjectName("widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout.addWidget(self.widget)
        self.widget_2 = QtWidgets.QWidget(Dialog)
        self.widget_2.setObjectName("widget_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.widget_2)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.checkBox_iscrowded = QtWidgets.QCheckBox(self.widget_2)
        self.checkBox_iscrowded.setObjectName("checkBox_iscrowded")
        self.horizontalLayout_2.addWidget(self.checkBox_iscrowded)
        spacerItem = QtWidgets.QSpacerItem(97, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.pushButton_cancel = QtWidgets.QPushButton(self.widget_2)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/icons/关闭_close-one.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_cancel.setIcon(icon)
        self.pushButton_cancel.setObjectName("pushButton_cancel")
        self.horizontalLayout_2.addWidget(self.pushButton_cancel)
        self.pushButton_apply = QtWidgets.QPushButton(self.widget_2)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/icons/校验_check-one.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_apply.setIcon(icon1)
        self.pushButton_apply.setObjectName("pushButton_apply")
        self.horizontalLayout_2.addWidget(self.pushButton_apply)
        self.verticalLayout.addWidget(self.widget_2)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        self.label_2.setText(_translate("Dialog", "category:"))
        self.label.setText(_translate("Dialog", "group:"))
        self.label_3.setText(_translate("Dialog", "note:"))
        self.lineEdit_note.setPlaceholderText(_translate("Dialog", "add extra note here"))
        self.label_4.setText(_translate("Dialog", "layer:"))
        self.checkBox_iscrowded.setText(_translate("Dialog", "is crowded"))
        self.pushButton_cancel.setText(_translate("Dialog", "cancel"))
        self.pushButton_apply.setText(_translate("Dialog", "apply"))
